@@ -13,18 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import os
-
-if os.name == 'nt':
-    from neutron.agent.windows import ip_lib
-    from neutron.conf.agent import windows
-    OPTS = windows.IP_LIB_OPTS_WINDOWS
-else:
-    from neutron.agent.linux import ip_lib
-    from neutron.conf.agent import linux
-    OPTS = linux.IP_LIB_OPTS_LINUX
+from neutron.agent.linux import ip_lib
+from neutron.conf.agent import linux
 
 
+OPTS = linux.IP_LIB_OPTS_LINUX
 IPWrapper = ip_lib.IPWrapper
 IPDevice = ip_lib.IPDevice
 
