@@ -188,13 +188,13 @@ class LinuxInterfaceDriver(interface.LinuxInterfaceDriver,
                              on-link route list
         """
         device = ip_lib.IPDevice(device_name, namespace=namespace)
-        new_onlink_cidrs = set(s['cidr'] for s in extra_subnets or [])
+        new_onlink_cidrs = {s['cidr'] for s in extra_subnets or []}
         preserve_ips = set(preserve_ips if preserve_ips else [])
 
         onlink = device.route.list_onlink_routes(constants.IP_VERSION_4)
         if is_ipv6:
             onlink += device.route.list_onlink_routes(constants.IP_VERSION_6)
-        existing_onlink_cidrs = set(r['cidr'] for r in onlink)
+        existing_onlink_cidrs = {r['cidr'] for r in onlink}
 
         for route in new_onlink_cidrs - existing_onlink_cidrs:
             LOG.debug('Adding onlink route (%s)', route)
@@ -245,8 +245,8 @@ class LinuxInterfaceDriver(interface.LinuxInterfaceDriver,
         """Configure handling of IPv6 Router Advertisements on an
         interface. See common/constants.py for possible values.
         """
-        cmd = ['net.ipv6.conf.%(dev)s.accept_ra=%(value)s' % {'dev': dev_name,
-                                                              'value': value}]
+        cmd = ['net.ipv6.conf.{dev}.accept_ra={value}'.format(dev=dev_name,
+                                                              value=value)]
         ip_lib.sysctl(cmd, namespace=namespace)
 
     @staticmethod
@@ -314,7 +314,7 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
     DEV_NAME_PREFIX = constants.TAP_DEVICE_PREFIX
 
     def __init__(self, conf, **kwargs):
-        super(OVSInterfaceDriver, self).__init__(conf, **kwargs)
+        super().__init__(conf, **kwargs)
         ovs_conf.register_ovs_agent_opts(self.conf)
         if self.conf.ovs_use_veth:
             self.DEV_NAME_PREFIX = 'ns-'
@@ -365,7 +365,7 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
         namespace_obj = ip_wrapper.ensure_namespace(namespace)
         for i in range(9):
             try:
-                namespace_obj.add_device_to_namespace(device, is_ovs_port=True)
+                namespace_obj.add_device_to_namespace(device)
                 break
             except ip_lib.NetworkInterfaceNotFound:
                 # NOTE(slaweq): if the exception was NetworkInterfaceNotFound
@@ -491,49 +491,3 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
         else:
             ns_dev = ip_lib.IPWrapper(namespace=namespace).device(device_name)
         ns_dev.link.set_mtu(mtu)
-
-
-class BridgeInterfaceDriver(LinuxInterfaceDriver):
-    """Driver for creating bridge interfaces."""
-
-    DEV_NAME_PREFIX = 'ns-'
-
-    def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
-        """Plugin the interface."""
-        ip = ip_lib.IPWrapper()
-
-        # Enable agent to define the prefix
-        tap_name = device_name.replace(prefix or self.DEV_NAME_PREFIX,
-                                       constants.TAP_DEVICE_PREFIX)
-        # Create ns_veth in a namespace if one is configured.
-        root_veth, ns_veth = ip.add_veth(tap_name, device_name,
-                                         namespace2=namespace)
-        root_veth.disable_ipv6()
-        ns_veth.link.set_address(mac_address)
-
-        if mtu:
-            self.set_mtu(device_name, mtu, namespace=namespace, prefix=prefix)
-        else:
-            LOG.warning("No MTU configured for port %s", port_id)
-
-        root_veth.link.set_up()
-        ns_veth.link.set_up()
-
-    def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
-        """Unplug the interface."""
-        device = ip_lib.IPDevice(device_name, namespace=namespace)
-        try:
-            device.link.delete()
-            LOG.debug("Unplugged interface '%s'", device_name)
-        except RuntimeError:
-            LOG.error("Failed unplugging interface '%s'",
-                      device_name)
-
-    def set_mtu(self, device_name, mtu, namespace=None, prefix=None):
-        tap_name = device_name.replace(prefix or self.DEV_NAME_PREFIX,
-                                       constants.TAP_DEVICE_PREFIX)
-        root_dev, ns_dev = _get_veth(
-            tap_name, device_name, namespace2=namespace)
-        root_dev.link.set_mtu(mtu)
-        ns_dev.link.set_mtu(mtu)
