@@ -13,13 +13,15 @@
 from neutron.api.rpc.callbacks import resource_manager
 
 
-#TODO(ajo): consider adding locking to _get_manager, it's
-#           safe for eventlet, but not for normal threading.
+# TODO(ajo): consider adding locking to _get_manager, it's
+#            safe for eventlet, but not for normal threading.
 def _get_manager():
     return resource_manager.ConsumerResourceCallbacksManager()
 
 
-def subscribe(callback, resource_type):
+def register(callback, resource_type):
+    # TODO(kevinbenton): remove this on debt collection
+    callback.__dict__['_ACCEPTS_CONTEXT'] = True
     _get_manager().register(callback, resource_type)
 
 
@@ -27,12 +29,17 @@ def unsubscribe(callback, resource_type):
     _get_manager().unregister(callback, resource_type)
 
 
-def push(resource_type, resource_list, event_type):
+def push(context, resource_type, resource_list, event_type):
     """Push resource list into all registered callbacks for the event type."""
 
     callbacks = _get_manager().get_callbacks(resource_type)
     for callback in callbacks:
-        callback(resource_list, event_type)
+        if callback._ACCEPTS_CONTEXT:
+            callback(context, resource_type, resource_list, event_type)
+        else:
+            # backwards compat for callback listeners that don't take
+            # context and resource_type
+            callback(resource_list, event_type)
 
 
 def clear():
