@@ -25,14 +25,13 @@ from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
-
-from neutron.common import constants as n_const
+from sqlalchemy import orm
 
 
 LOG = logging.getLogger(__name__)
 
 
-class L3RpcCallback(object):
+class L3RpcCallback:
     """L3 agent RPC callback in plugin implementations."""
 
     # 1.0 L3PluginApi BASE_RPC_API_VERSION
@@ -48,7 +47,9 @@ class L3RpcCallback(object):
     # 1.9 Added get_router_ids
     # 1.10 Added update_all_ha_network_port_statuses
     # 1.11 Added get_host_ha_router_count
-    target = oslo_messaging.Target(version='1.11')
+    # 1.12 Added get_networks
+    # 1.13 Removed process_prefix_update
+    target = oslo_messaging.Target(version='1.13')
 
     @property
     def plugin(self):
@@ -76,7 +77,7 @@ class L3RpcCallback(object):
         calling update_device_up.
         """
         if not extensions.is_extension_supported(
-            self.plugin, constants.PORT_BINDING_EXT_ALIAS):
+                self.plugin, constants.PORT_BINDING_EXT_ALIAS):
             return
         device_filter = {
             'device_owner': [constants.DEVICE_OWNER_ROUTER_HA_INTF],
@@ -90,8 +91,13 @@ class L3RpcCallback(object):
                   "setting HA network ports %(ha_ports)s status to DOWN.",
                   {"host": host, "ha_ports": ha_ports})
         for p in ha_ports:
-            self.plugin.update_port(
-                context, p, {'port': {'status': constants.PORT_STATUS_DOWN}})
+            try:
+                self.plugin.update_port(
+                    context, p,
+                    {'port': {'status': constants.PORT_STATUS_DOWN}})
+            except (orm.exc.StaleDataError, orm.exc.ObjectDeletedError,
+                    exceptions.PortNotFound):
+                pass
 
     def get_router_ids(self, context, host):
         """Returns IDs of routers scheduled to l3 agent on <host>
@@ -117,26 +123,30 @@ class L3RpcCallback(object):
         router_ids = kwargs.get('router_ids')
         host = kwargs.get('host')
         context = neutron_context.get_admin_context()
+        LOG.debug('Sync routers for ids %(router_ids)s in %(host)s',
+                  {'router_ids': router_ids,
+                   'host': host})
         routers = self._routers_to_sync(context, router_ids, host)
         if extensions.is_extension_supported(
-            self.plugin, constants.PORT_BINDING_EXT_ALIAS):
+                self.plugin, constants.PORT_BINDING_EXT_ALIAS):
             self._ensure_host_set_on_ports(context, host, routers)
             # refresh the data structure after ports are bound
             routers = self._routers_to_sync(context, router_ids, host)
         pf_plugin = directory.get_plugin(plugin_constants.PORTFORWARDING)
         if pf_plugin:
             pf_plugin.sync_port_forwarding_fip(context, routers)
+        LOG.debug('The sync data for ids %(router_ids)s in %(host)s is: '
+                  '%(routers)s', {'router_ids': router_ids,
+                                  'host': host,
+                                  'routers': routers})
         return routers
 
     def _routers_to_sync(self, context, router_ids, host=None):
         if extensions.is_extension_supported(
-            self.l3plugin, constants.L3_AGENT_SCHEDULER_EXT_ALIAS):
-            routers = (
-                self.l3plugin.list_active_sync_routers_on_active_l3_agent(
-                    context, host, router_ids))
-        else:
-            routers = self.l3plugin.get_sync_data(context, router_ids)
-        return routers
+                self.l3plugin, constants.L3_AGENT_SCHEDULER_EXT_ALIAS):
+            return self.l3plugin.list_active_sync_routers_on_active_l3_agent(
+                context, host, router_ids)
+        return self.l3plugin.get_sync_data(context, router_ids)
 
     def _ensure_host_set_on_ports(self, context, host, routers):
         for router in routers:
@@ -151,7 +161,7 @@ class L3RpcCallback(object):
                                               router.get('gw_port'),
                                               router['id'],
                                               ha_router_port=router.get('ha'))
-                for p in router.get(n_const.SNAT_ROUTER_INTF_KEY, []):
+                for p in router.get(constants.SNAT_ROUTER_INTF_KEY, []):
                     self._ensure_host_set_on_port(
                         context, gw_port_host, p, router['id'],
                         ha_router_port=router.get('ha'))
@@ -180,9 +190,9 @@ class L3RpcCallback(object):
             portbindings.VIF_TYPE_BINDING_FAILED,
             portbindings.VIF_TYPE_UNBOUND)
         if (port and host is not None and
-            (port.get('device_owner') !=
-             constants.DEVICE_OWNER_DVR_INTERFACE and
-             port.get(portbindings.HOST_ID) != host or not_bound)):
+                (port.get('device_owner') !=
+                 constants.DEVICE_OWNER_DVR_INTERFACE and
+                 port.get(portbindings.HOST_ID) != host or not_bound)):
 
             # Ports owned by non-HA routers are bound again if they're
             # already bound but the router moved to another host.
@@ -212,22 +222,21 @@ class L3RpcCallback(object):
                     active_host = (
                         self.l3plugin.get_active_host_for_ha_router(
                             context, router_id))
-                    if active_host:
-                        host = active_host
-                    # If there is currently no active router instance (For
-                    # example it's a new router), the host that requested
-                    # the routers (Essentially a random host) will do. The
-                    # port binding will be corrected when an active is
-                    # elected.
+                    if not active_host:
+                        LOG.debug("Router %(router)s is not active on any "
+                                  "host. Port %(port)s will not be updated "
+                                  "now.",
+                                  {'router': router_id, 'port': port['id']})
+                        return
                     try:
                         LOG.debug("Updating router %(router)s port %(port)s "
                                   "binding host %(host)s",
                                   {"router": router_id, "port": port['id'],
-                                   "host": host})
+                                   "host": active_host})
                         self.plugin.update_port(
                             context,
                             port['id'],
-                            {'port': {portbindings.HOST_ID: host}})
+                            {'port': {portbindings.HOST_ID: active_host}})
                     except exceptions.PortNotFound:
                         LOG.debug("Port %(port)s not found while updating "
                                   "agent binding for router %(router)s.",
@@ -244,18 +253,6 @@ class L3RpcCallback(object):
                 {'port': {portbindings.HOST_ID: host,
                           'device_id': router_id}})
 
-    def get_external_network_id(self, context, **kwargs):
-        """Get one external network id for l3 agent.
-
-        l3 agent expects only one external network when it performs
-        this query.
-        """
-        context = neutron_context.get_admin_context()
-        net_id = self.plugin.get_external_network_id(context)
-        LOG.debug("External network ID returned to l3 agent: %s",
-                  net_id)
-        return net_id
-
     def get_service_plugin_list(self, context, **kwargs):
         return directory.get_plugins().keys()
 
@@ -265,7 +262,7 @@ class L3RpcCallback(object):
     @db_api.retry_db_errors
     def update_floatingip_statuses(self, context, router_id, fip_statuses):
         """Update operational status for a floating IP."""
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             for (floatingip_id, status) in fip_statuses.items():
                 LOG.debug("New status for floating IP %(floatingip_id)s: "
                           "%(status)s", {'floatingip_id': floatingip_id,
@@ -296,8 +293,8 @@ class L3RpcCallback(object):
         """DVR: RPC called by dvr-agent to get all ports for subnet."""
         subnet_id = kwargs.get('subnet_id')
         LOG.debug("DVR: subnet_id: %s", subnet_id)
-        filters = {'fixed_ips': {'subnet_id': [subnet_id]}}
-        return self.plugin.get_ports(context, filters=filters)
+        return self.l3plugin.get_ports_under_dvr_connected_subnet(
+            context, subnet_id)
 
     @db_api.retry_db_errors
     def get_agent_gateway_port(self, context, **kwargs):
@@ -331,17 +328,6 @@ class L3RpcCallback(object):
         LOG.debug('Updating HA routers states on host %s: %s', host, states)
         self.l3plugin.update_routers_states(context, states, host)
 
-    def process_prefix_update(self, context, **kwargs):
-        subnets = kwargs.get('subnets')
-
-        updated_subnets = []
-        for subnet_id, prefix in subnets.items():
-            updated_subnets.append(self.plugin.update_subnet(
-                                        context,
-                                        subnet_id,
-                                        {'subnet': {'cidr': prefix}}))
-        return updated_subnets
-
     @db_api.retry_db_errors
     def delete_agent_gateway_port(self, context, **kwargs):
         """Delete Floatingip agent gateway port."""
@@ -350,3 +336,11 @@ class L3RpcCallback(object):
         admin_ctx = neutron_context.get_admin_context()
         self.l3plugin.delete_floatingip_agent_gateway_port(
             admin_ctx, host, network_id)
+
+    def get_networks(self, context, filters=None, fields=None):
+        """Retrieve and return a list of networks."""
+        # NOTE(adrianc): This RPC is being used by out of tree interface
+        # drivers, MultiInterfaceDriver and IPoIBInterfaceDriver, located in
+        # networking-mlnx.
+        return self.plugin.get_networks(
+            context, filters=filters, fields=fields)
