@@ -10,10 +10,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import setproctitle
+# NOTE(haleyb): remove once the default backend is ``BackendType.THREADING``
+import oslo_service.backend as service
+service.init_backend(service.BackendType.THREADING)
 
-from neutron.agent.ovn import metadata_agent
-from neutron_lib import constants
+# pylint: disable=wrong-import-position
+import setproctitle  # noqa: E402
+
+from neutron.agent.ovn import metadata_agent  # noqa: E402
+from neutron_lib import constants  # noqa: E402
 
 
 def main():
