@@ -26,7 +26,8 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_middleware import cors
-from oslo_policy import opts
+from oslo_reports import guru_meditation_report as gmr
+from oslo_reports import opts as gmr_opts
 from oslo_service import wsgi
 
 from neutron._i18n import _
@@ -115,7 +116,7 @@ def setup_logging():
     # that Neutron needs.
     logging.set_defaults(default_log_levels=logging.get_default_log_levels() +
                          EXTRA_LOG_LEVEL_DEFAULTS)
-    logging.setup(cfg.CONF, product_name)
+    logging.setup(cfg.CONF, product_name, fix_eventlet=False)
     LOG.info("Logging enabled!")
     LOG.info("%(prog)s version %(version)s",
              {'prog': sys.argv[0],
@@ -123,6 +124,14 @@ def setup_logging():
     LOG.debug("command line: %s", " ".join(sys.argv))
 
 
+def setup_gmr():
+    """Sets up Guru Meditation Report(GMR) generation."""
+    gmr_opts.set_defaults(cfg.CONF)
+    _version_string = version.version_info.release_string()
+    gmr.TextGuruMeditation.setup_autorun(version=_version_string,
+                                         conf=cfg.CONF)
+
+
 def reset_service():
     # Reset worker in case SIGHUP is called.
     # Note that this is called only in case a service is running in
@@ -150,11 +159,6 @@ def set_config_defaults():
     """This method updates all configuration default values."""
     set_cors_middleware_defaults()
 
-    # TODO(gmann): Remove setting the default value of config policy_file
-    # once oslo_policy change the default value to 'policy.yaml'.
-    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
-    opts.set_defaults(cfg.CONF, policy.DEFAULT_POLICY_FILE)
-
 
 def set_cors_middleware_defaults():
     """Update default configuration options for oslo.middleware."""
