@@ -207,7 +207,10 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
             return []
 
         admin_context = context.elevated()
-        port_host = deleted_port[portbindings.HOST_ID]
+        port_host = deleted_port.get(portbindings.HOST_ID)
+        if not port_host:
+            return []
+
         subnet_ids = [ip['subnet_id'] for ip in deleted_port['fixed_ips']]
         router_ids = self.get_dvr_routers_by_subnet_ids(admin_context,
                                                         subnet_ids)
@@ -300,7 +303,7 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
 
     def get_hosts_to_notify(self, context, router_id):
         """Returns all hosts to send notification about router update"""
-        hosts = super(L3_DVRsch_db_mixin, self).get_hosts_to_notify(
+        hosts = super().get_hosts_to_notify(
             context, router_id)
         router = self.get_router(context.elevated(), router_id)
         if router.get('distributed', False):
@@ -321,7 +324,8 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         get a set of hosts where all dvr serviceable ports on those subnets
         are bound
         """
-        subnet_ids = self.get_subnet_ids_on_router(context, router_id)
+        subnet_ids = self.get_subnet_ids_on_router(context, router_id,
+                                                   keep_gateway_port=False)
         hosts = self._get_dvr_hosts_for_subnets(context, subnet_ids)
         LOG.debug('Hosts for router %s: %s', router_id, hosts)
         return hosts
@@ -414,9 +418,8 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
 
     def _get_router_ids_for_agent(self, context, agent_db, router_ids,
                                   with_dvr=True):
-        result_set = set(super(L3_DVRsch_db_mixin,
-                               self)._get_router_ids_for_agent(
-            context, agent_db, router_ids, with_dvr))
+        result_set = set(super()._get_router_ids_for_agent(
+                                   context, agent_db, router_ids, with_dvr))
         if not with_dvr:
             return result_set
         LOG.debug("Routers %(router_ids)s bound to L3 agent in host %(host)s",
@@ -431,9 +434,9 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         # dvr routers are not explicitly scheduled to agents on hosts with
         # dvr serviceable ports, so need special handling
         if (self._get_agent_mode(agent_db) in
-            [n_const.L3_AGENT_MODE_DVR,
-             n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL,
-             n_const.L3_AGENT_MODE_DVR_SNAT]):
+                [n_const.L3_AGENT_MODE_DVR,
+                 n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL,
+                 n_const.L3_AGENT_MODE_DVR_SNAT]):
             dvr_routers = self._get_dvr_router_ids_for_host(context,
                                                             agent_db['host'])
             if not router_ids:
@@ -444,10 +447,10 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
                         context, router_id, keep_gateway_port=False)
                     if (subnet_ids and (
                             self._check_dvr_serviceable_ports_on_host(
-                                    context, agent_db['host'],
-                                    list(subnet_ids)) or
+                                context, agent_db['host'],
+                                list(subnet_ids)) or
                             self._is_router_related_to_dvr_routers(
-                                    context, router_id, dvr_routers))):
+                                context, router_id, dvr_routers))):
                         result_set.add(router_id)
 
             LOG.debug("Routers %(router_ids)s are scheduled or have "
@@ -518,7 +521,7 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
                                           dvr_routers):
         related_routers = self._get_other_dvr_router_ids_connected_router(
             context, router_id)
-        return any([r in dvr_routers for r in related_routers])
+        return any(r in dvr_routers for r in related_routers)
 
 
 def _dvr_handle_unbound_allowed_addr_pair_add(
@@ -553,7 +556,7 @@ def _notify_port_delete(event, resource, trigger, payload):
     context = payload.context
     port = payload.latest_state
     get_related_hosts_info = payload.metadata.get(
-                                 "get_related_hosts_info", True)
+        "get_related_hosts_info", True)
     l3plugin = directory.get_plugin(plugin_constants.L3)
     if port:
         port_host = port.get(portbindings.HOST_ID)
@@ -601,7 +604,7 @@ def _notify_l3_agent_port_update(resource, event, trigger, payload):
             dest_host = new_port_profile.get('migrating_to')
         if is_new_port_binding_changed or is_bound_port_moved or dest_host:
             fips = l3plugin._get_floatingips_by_port_id(
-                    context, port_id=original_port['id'])
+                context, port_id=original_port['id'])
             fip = fips[0] if fips else None
             if fip:
                 fip_router_id = fip['router_id']
@@ -666,7 +669,7 @@ def _notify_l3_agent_port_update(resource, event, trigger, payload):
                     _dvr_handle_unbound_allowed_addr_pair_add(
                         l3plugin, context, new_port, address_pair)
                 return
-            elif original_port_state:
+            if original_port_state:
                 # Case were we deactivate the port from active state.
                 for address_pair in allowed_address_pairs_list:
                     _dvr_handle_unbound_allowed_addr_pair_del(
