@@ -41,6 +41,8 @@ RELEASE_2023_1 = '2023.1'
 RELEASE_2023_2 = '2023.2'
 RELEASE_2024_1 = '2024.1'
 RELEASE_2024_2 = '2024.2'
+RELEASE_2025_1 = '2025.1'
+RELEASE_2025_2 = '2025.2'
 
 NEUTRON_MILESTONES = [
     # earlier milestones were not tagged
@@ -58,8 +60,12 @@ NEUTRON_MILESTONES = [
     WALLABY,
     XENA,
     YOGA,
+    ZED,
     RELEASE_2023_1,
     RELEASE_2023_2,
+    RELEASE_2024_1,
+    RELEASE_2024_2,
+    RELEASE_2025_1,
     # Do not add the milestone until the end of the release
 ]
 
@@ -112,6 +118,21 @@ def schema_has_column(table_name, column_name):
                            insp.get_columns(table_name)]
 
 
+@raise_if_offline
+def create_table_if_not_exists(table_name, *args, **kwargs):
+    table = None
+    if not schema_has_table(table_name):
+        table = op.create_table(table_name, *args, **kwargs)
+    return table
+
+
+@raise_if_offline
+def add_column_if_not_exists(table_name, column, **kwargs):
+    """Add column only if it not exists in the schema."""
+    if not schema_has_column(table_name, column.name):
+        op.add_column(table_name, column, **kwargs)
+
+
 @raise_if_offline
 def alter_column_if_exists(table_name, column_name, **kwargs):
     """Alter a column only if it exists in the schema."""
@@ -132,9 +153,7 @@ def rename_table_if_exists(old_table_name, new_table_name):
 
 
 def alter_enum_add_value(table, column, enum, nullable, server_default=None):
-    '''If we need to expand Enum values for some column - for PostgreSQL this
-    can be done with ALTER TYPE function. For MySQL, it can be done with
-    ordinary alembic alter_column function.
+    '''Expand Enum values for a column.
 
     :param table:table name
     :param column: column name
@@ -142,86 +161,9 @@ def alter_enum_add_value(table, column, enum, nullable, server_default=None):
     :param nullable: existing nullable for column.
     :param server_default: existing or new server_default for the column
     '''
-
-    bind = op.get_bind()
-    engine = bind.engine
-    if engine.name == 'postgresql':
-        values = {'name': enum.name,
-                  'values': ", ".join("'" + i + "'" for i in enum.enums),
-                  'column': column,
-                  'table': table,
-                  'server_default': server_default}
-        if server_default is not None:
-            op.execute("ALTER TABLE %(table)s ALTER COLUMN %(column)s"
-                       " DROP DEFAULT" % values)
-        op.execute("ALTER TYPE %(name)s rename to old_%(name)s" % values)
-        op.execute("CREATE TYPE %(name)s AS enum (%(values)s)" % values)
-        op.execute("ALTER TABLE %(table)s ALTER COLUMN %(column)s TYPE "
-                   "%(name)s USING %(column)s::text::%(name)s " % values)
-        if server_default is not None:
-            op.execute("ALTER TABLE %(table)s ALTER COLUMN %(column)s"
-                       " SET DEFAULT '%(server_default)s'" % values)
-        op.execute("DROP TYPE old_%(name)s" % values)
-    else:
-        op.alter_column(table, column, type_=enum,
-                        existing_nullable=nullable,
-                        server_default=server_default)
-
-
-def alter_enum(table, column, enum_type, nullable,
-               server_default=None, do_drop=True,
-               do_rename=True, do_create=True):
-    """Alter a enum type column.
-    Set the do_xx parameters only when the modified enum type
-    is used by multiple columns. Else don't provide these
-    parameters.
-
-    :param server_default: existing or new server_default for the column
-    :param do_drop: set to False when modified column is
-    not the last one use this enum
-    :param do_rename: set to False when modified column is
-    not the first one use this enum
-    :param do_create: set to False when modified column is
-    not the first one use this enum
-    """
-    bind = op.get_bind()
-    engine = bind.engine
-    if engine.name == 'postgresql':
-        values = {'table': table,
-                  'column': column,
-                  'name': enum_type.name}
-        if do_rename:
-            op.execute("ALTER TYPE %(name)s RENAME TO old_%(name)s" % values)
-        if do_create:
-            enum_type.create(bind, checkfirst=False)
-        op.execute("ALTER TABLE %(table)s RENAME COLUMN %(column)s TO "
-                   "old_%(column)s" % values)
-        op.add_column(table, sa.Column(column, enum_type, nullable=nullable,
-                                       server_default=server_default))
-        op.execute("UPDATE %(table)s SET %(column)s = "  # nosec
-                   "old_%(column)s::text::%(name)s" % values)
-        op.execute("ALTER TABLE %(table)s DROP COLUMN old_%(column)s" % values)
-        if do_drop:
-            op.execute("DROP TYPE old_%(name)s" % values)
-    else:
-        op.alter_column(table, column, type_=enum_type,
-                        existing_nullable=nullable,
-                        server_default=server_default)
-
-
-def create_table_if_not_exist_psql(table_name, values):
-    if op.get_bind().engine.dialect.server_version_info < (9, 1, 0):
-        op.execute("CREATE LANGUAGE plpgsql")
-    op.execute("CREATE OR REPLACE FUNCTION execute(TEXT) RETURNS VOID AS $$"
-               "BEGIN EXECUTE $1; END;"
-               "$$ LANGUAGE plpgsql STRICT;")
-    op.execute("CREATE OR REPLACE FUNCTION table_exist(TEXT) RETURNS bool as "
-               "$$ SELECT exists(select 1 from pg_class where relname=$1);"
-               "$$ language sql STRICT;")
-    op.execute("SELECT execute($$CREATE TABLE %(name)s %(columns)s $$) "
-               "WHERE NOT table_exist(%(name)r);" %
-               {'name': table_name,
-                'columns': values})
+    op.alter_column(table, column, type_=enum,
+                    existing_nullable=nullable,
+                    server_default=server_default)
 
 
 def get_unique_constraints_map(table):
