@@ -15,6 +15,7 @@
 import abc
 import datetime
 
+from neutron_lib import constants as n_const
 from neutron_lib import context as neutron_context
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
@@ -34,6 +35,7 @@ from neutron.common.ovn import hash_ring_manager
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db
+from neutron.objects import router as router_obj
 from neutron.plugins.ml2.drivers.ovn.agent import neutron_agent as n_agent
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.extensions import \
     placement
@@ -43,13 +45,13 @@ CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 
-class BaseEvent(row_event.RowEvent):
-    table = None
+class BaseEvent(row_event.RowEvent, metaclass=abc.ABCMeta):
+    table: str
     events = tuple()
 
     def __init__(self):
         self.event_name = self.__class__.__name__
-        super(BaseEvent, self).__init__(self.events, self.table, None)
+        super().__init__(self.events, self.table, None)
 
     @abc.abstractmethod
     def match_fn(self, event, row, old=None):
@@ -73,7 +75,7 @@ class ChassisEvent(row_event.RowEvent):
         self.l3_plugin = directory.get_plugin(constants.L3)
         table = 'Chassis'
         events = (self.ROW_CREATE, self.ROW_UPDATE, self.ROW_DELETE)
-        super(ChassisEvent, self).__init__(events, table, None)
+        super().__init__(events, table, None)
         self.event_name = 'ChassisEvent'
 
     def _get_ha_chassis_groups_within_azs(self, az_hints):
@@ -149,7 +151,7 @@ class ChassisEvent(row_event.RowEvent):
                             self.driver.nb_ovn.ha_chassis_group_del_chassis(
                                 hcg.name, row.name, if_exists=True))
                 return
-            elif not is_gw_chassis and is_old_gw:
+            if not is_gw_chassis and is_old_gw:
                 # Chassis is not a gateway anymore, treat it as deletion
                 event = self.ROW_DELETE
             elif is_gw_chassis and not is_old_gw:
@@ -256,7 +258,7 @@ class PortBindingChassisUpdateEvent(row_event.RowEvent):
         self.driver = driver
         table = 'Port_Binding'
         events = (self.ROW_UPDATE,)
-        super(PortBindingChassisUpdateEvent, self).__init__(
+        super().__init__(
             events, table, None)
         self.event_name = self.__class__.__name__
 
@@ -294,6 +296,7 @@ class PortBindingChassisUpdateEvent(row_event.RowEvent):
 
 class ChassisAgentEvent(BaseEvent):
     GLOBAL = True
+    table = 'Chassis_Private'
 
     # NOTE (twilson) Do not run new transactions out of a GLOBAL Event since
     # it will be running on every single process, and you almost certainly
@@ -302,16 +305,6 @@ class ChassisAgentEvent(BaseEvent):
         self.driver = driver
         super().__init__()
 
-    @property
-    def table(self):
-        # It probably doesn't matter, but since agent_chassis_table changes
-        # in post_fork_initialize(), resolve this at runtime
-        return self.driver.agent_chassis_table
-
-    @table.setter
-    def table(self, value):
-        pass
-
 
 class ChassisAgentDownEvent(ChassisAgentEvent):
     events = (BaseEvent.ROW_DELETE,)
@@ -346,9 +339,7 @@ class ChassisAgentWriteEvent(ChassisAgentEvent):
         # On updates to Chassis_Private because the Chassis has been deleted,
         # don't update the AgentCache. We use chassis_private.chassis to return
         # data about the agent.
-        return event == self.ROW_CREATE or (
-            hasattr(old, 'nb_cfg') and not
-            (self.table == 'Chassis_Private' and not row.chassis))
+        return event == self.ROW_CREATE or hasattr(old, 'nb_cfg')
 
     def run(self, event, row, old):
         n_agent.AgentCache().update(ovn_const.OVN_CONTROLLER_AGENT, row,
@@ -361,35 +352,52 @@ class ChassisAgentTypeChangeEvent(ChassisEvent):
     events = (BaseEvent.ROW_UPDATE,)
 
     def match_fn(self, event, row, old=None):
-        # NOTE(ralonsoh): LP#1990229 to be removed when min OVN version is
-        # 22.09
-        other_config = ('other_config' if hasattr(row, 'other_config') else
-                        'external_ids')
-        if not getattr(old, other_config, False):
+        try:
+            return row.other_config.get('ovn-cms-options', []) != (
+                old.other_config.get('ovn-cms-options', []))
+        except AttributeError:
+            # No change to other_config
             return False
-        chassis = n_agent.NeutronAgent.chassis_from_private(row)
-        new_other_config = utils.get_ovn_chassis_other_config(chassis)
-        old_other_config = utils.get_ovn_chassis_other_config(old)
-        agent_type_change = new_other_config.get('ovn-cms-options', []) != (
-            old_other_config.get('ovn-cms-options', []))
-        return agent_type_change
 
     def run(self, event, row, old):
-        n_agent.AgentCache().update(ovn_const.OVN_CONTROLLER_AGENT, row,
-                                    clear_down=event == self.ROW_CREATE)
+        # the row is in the Chassis table but the agent cache uses
+        # Chassis_Private rows
+        try:
+            ch_private = self.driver.sb_ovn.db_find_rows(
+                'Chassis_Private', ('chassis', '=', row.uuid)).execute(
+                    check_error=True)[0]
+        except IndexError:
+            # The chassis private row was not found, this should never happen
+            LOG.error("The Chassis_Private row for Chassis %s was not found.",
+                      row.uuid)
+            return
+        # The passed agent type is significant to the method obtaining the
+        # agent chassis id. This method is the same for both Gateway and
+        # Controller agent types so the type below can be either.
+        n_agent.AgentCache().update(ovn_const.OVN_CONTROLLER_AGENT, ch_private)
 
 
-class ChassisMetadataAgentWriteEvent(ChassisAgentEvent):
+class ChassisOVNAgentWriteEvent(ChassisAgentEvent):
     events = (BaseEvent.ROW_CREATE, BaseEvent.ROW_UPDATE)
 
     @staticmethod
-    def _metadata_nb_cfg(row):
+    def _agent_sb_cfg(row):
+        external_ids = row.external_ids
+        # Try the OVN agent SB cfg first, then fallback to the OVN Metadata
+        # agent
+        ovn_sb_cfg = external_ids.get(ovn_const.OVN_AGENT_NEUTRON_SB_CFG_KEY)
+        if ovn_sb_cfg:
+            return int(ovn_sb_cfg)
+        # NOTE(ralonsoh): to remove when the OVN Metadata agent is removed.
         return int(
-            row.external_ids.get(ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY, -1))
+            external_ids.get(ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY, -1))
 
     @staticmethod
     def agent_id(row):
-        return row.external_ids.get(ovn_const.OVN_AGENT_METADATA_ID_KEY)
+        external_ids = row.external_ids
+        # NOTE(ralonsoh): to update when the OVN Metadata agent is removed.
+        return (external_ids.get(ovn_const.OVN_AGENT_NEUTRON_ID_KEY) or
+                external_ids.get(ovn_const.OVN_AGENT_METADATA_ID_KEY))
 
     def match_fn(self, event, row, old=None):
         if not self.agent_id(row):
@@ -397,19 +405,35 @@ class ChassisMetadataAgentWriteEvent(ChassisAgentEvent):
             return False
         if event == self.ROW_CREATE:
             return True
+
+        # On updates to Chassis_Private because the Chassis has been
+        # deleted, don't update the AgentCache. We use
+        # chassis_private.chassis to return data about the agent.
+        if not getattr(row, 'chassis', None):
+            return False
+
+        # Check if both rows have external_ids before comparing nb_cfg
+        if not (hasattr(old, 'external_ids') and row.external_ids):
+            return False
+
         try:
-            # On updates to Chassis_Private because the Chassis has been
-            # deleted, don't update the AgentCache. We use
-            # chassis_private.chassis to return data about the agent.
-            if self.table == 'Chassis_Private' and not row.chassis:
-                return False
-            return self._metadata_nb_cfg(row) != self._metadata_nb_cfg(old)
-        except (AttributeError, KeyError):
+            # Cache the nb_cfg values to avoid duplicate calculations
+            row_sb_cfg = self._agent_sb_cfg(row)
+            old_sb_cfg = self._agent_sb_cfg(old)
+            return row_sb_cfg != old_sb_cfg
+        except (AttributeError, KeyError, TypeError):
             return False
 
     def run(self, event, row, old):
-        n_agent.AgentCache().update(ovn_const.OVN_METADATA_AGENT, row,
-                                    clear_down=True)
+        external_ids = row.external_ids
+        if external_ids.get(ovn_const.OVN_AGENT_NEUTRON_ID_KEY):
+            n_agent.AgentCache().update(ovn_const.OVN_NEUTRON_AGENT, row,
+                                        clear_down=True)
+        else:
+            # NOTE(ralonsoh): to remove when the OVN Metadata agent is
+            # removed.
+            n_agent.AgentCache().update(ovn_const.OVN_METADATA_AGENT, row,
+                                        clear_down=True)
 
 
 class PortBindingChassisEvent(row_event.RowEvent):
@@ -491,7 +515,7 @@ class LogicalSwitchPortUpdateUpEvent(row_event.RowEvent):
         self.driver = driver
         table = 'Logical_Switch_Port'
         events = (self.ROW_UPDATE,)
-        super(LogicalSwitchPortUpdateUpEvent, self).__init__(
+        super().__init__(
             events, table, None)
         self.event_name = 'LogicalSwitchPortUpdateUpEvent'
 
@@ -521,7 +545,7 @@ class LogicalSwitchPortUpdateDownEvent(row_event.RowEvent):
         self.driver = driver
         table = 'Logical_Switch_Port'
         events = (self.ROW_UPDATE,)
-        super(LogicalSwitchPortUpdateDownEvent, self).__init__(
+        super().__init__(
             events, table, None)
         self.event_name = 'LogicalSwitchPortUpdateDownEvent'
 
@@ -543,6 +567,63 @@ class LogicalSwitchPortUpdateDownEvent(row_event.RowEvent):
         self.driver.set_port_status_down(row.name)
 
 
+class LogicalSwitchPortUpdateLogicalRouterPortEvent(row_event.RowEvent):
+    """Row update event - Logical_Switch_Port, that updates the sibling LRP"""
+    def __init__(self, driver):
+        self.driver = driver
+        table = 'Logical_Switch_Port'
+        events = (self.ROW_UPDATE,)
+        super().__init__(events, table, None)
+        self.event_name = 'LogicalSwitchPortUpdateLogicalRouterPortEvent'
+        self.l3_plugin = directory.get_plugin(constants.L3)
+        self.admin_context = neutron_context.get_admin_context()
+
+    def match_fn(self, event, row, old):
+        device_id = row.external_ids.get(ovn_const.OVN_DEVID_EXT_ID_KEY)
+        device_owner = row.external_ids.get(
+            ovn_const.OVN_DEVICE_OWNER_EXT_ID_KEY)
+        if (not device_id or
+                device_owner not in n_const.ROUTER_INTERFACE_OWNERS):
+            # This LSP does not belong to a router.
+            return False
+
+        lrp_name = utils.ovn_lrouter_port_name(row.name)
+        if not self.driver.nb_ovn.lookup('Logical_Router_Port', lrp_name,
+                                         default=None):
+            # The LRP has not been created yet.
+            return False
+
+        # TODO(ralonsoh): store the router "flavor_id" in the LSP.external_ids
+        # or the LRP.external_ids (better the second).
+        router = router_obj.Router.get_object(self.admin_context, id=device_id,
+                                              fields=('flavor_id', ))
+        if (utils.is_lsp_router_port(lsp=row) and
+                router and
+                utils.is_ovn_provider_router(router)):
+            return True
+        return False
+
+    def run(self, event, row, old):
+        # In some cases, it is possible for the logical switch port to be
+        # already removed from db by some other concurrent event when this
+        # method is called. Therefore, use get_ports to just query for this
+        # port instead of directly trying to get it from db causing not
+        # found exception.
+        ports = self.driver._plugin.get_ports(
+            self.admin_context,
+            filters={'id': [row.name]})
+        if ports:
+            self.l3_plugin._ovn_client.update_router_port(
+                self.admin_context,
+                ports[0])
+        else:
+            LOG.debug('Port %(port_id)s not found when '
+                      'run of %(event_name)s was called. '
+                      'Router port was not updated.',
+                      {'port_id': row.name,
+                       'event_name': self.event_name})
+
+
 class PortBindingUpdateVirtualPortsEvent(row_event.RowEvent):
     """Row update event - Port_Binding for virtual ports
 
@@ -611,7 +692,7 @@ class FIPAddDeleteEvent(row_event.RowEvent):
         self.driver = driver
         table = 'NAT'
         events = (self.ROW_CREATE, self.ROW_DELETE)
-        super(FIPAddDeleteEvent, self).__init__(
+        super().__init__(
             events, table, (('type', '=', 'dnat_and_snat'),))
         self.event_name = 'FIPAddDeleteEvent'
 
@@ -623,10 +704,58 @@ class FIPAddDeleteEvent(row_event.RowEvent):
         self.driver.delete_mac_binding_entries(row.external_ip)
 
 
+class HAChassisGroupRouterEvent(row_event.RowEvent):
+    """Row update event - the HA_Chassis list changes in a router HCG
+
+    When the HA_Chassis list changes (a chassis has been added, deleted or
+    updated), those routers with a HA_Chassis_Group related should update the
+    "LR.options.chassis" value.
+    """
+    def __init__(self, driver):
+        self.driver = driver
+        table = 'HA_Chassis_Group'
+        events = (self.ROW_UPDATE,)
+        super().__init__(events, table, None)
+        self.event_name = 'HAChassisGroupRouterEvent'
+
+    def match_fn(self, event, row, old):
+        if ovn_const.OVN_ROUTER_ID_EXT_ID_KEY not in row.external_ids:
+            # This is not a router "HA_Chassis_Group".
+            return False
+        if hasattr(old, 'ha_chassis'):
+            # "HA_Chassis_Group" has been assigned to a router or there are
+            # changes in the "ha_chassis" list.
+            return True
+        return False
+
+    def run(self, event, row, old):
+        router_id = row.external_ids[ovn_const.OVN_ROUTER_ID_EXT_ID_KEY]
+        router_name = utils.ovn_name(router_id)
+        if not row.ha_chassis:
+            # No GW chassis are present in the environment.
+            self.driver.nb_ovn.db_remove(
+                'Logical_Router', router_name, 'options', 'chassis',
+                if_exists=True).execute(check_error=True)
+            LOG.info('Router %s is not pinned to any gateway chassis',
+                     router_id)
+            return
+
+        try:
+            highest_prio_hc = max(row.ha_chassis, key=lambda hc: hc.priority)
+        except ValueError:
+            highest_prio_hc = None
+
+        if highest_prio_hc:
+            options = {'chassis': highest_prio_hc.chassis_name}
+            self.driver.nb_ovn.db_set(
+                'Logical_Router', router_name, ('options', options)).execute(
+                check_error=True)
+
+
 class OvnDbNotifyHandler(row_event.RowEventHandler):
     def __init__(self, driver):
         self.driver = driver
-        super(OvnDbNotifyHandler, self).__init__()
+        super().__init__()
         try:
             self._lock = self._RowEventHandler__lock
             self._watched_events = self._RowEventHandler__watched_events
@@ -651,7 +780,7 @@ class Ml2OvnIdlBase(connection.OvsdbIdl):
     def __init__(self, remote, schema, probe_interval=(), **kwargs):
         if probe_interval == ():  # None is a valid value to pass
             probe_interval = ovn_conf.get_ovn_ovsdb_probe_interval()
-        super(Ml2OvnIdlBase, self).__init__(
+        super().__init__(
             remote, schema, probe_interval=probe_interval, **kwargs)
 
     def set_table_condition(self, table_name, condition):
@@ -667,7 +796,7 @@ class Ml2OvnIdlBase(connection.OvsdbIdl):
 class BaseOvnIdl(Ml2OvnIdlBase):
     def __init__(self, remote, schema, **kwargs):
         self.notify_handler = row_event.RowEventHandler()
-        super(BaseOvnIdl, self).__init__(remote, schema, **kwargs)
+        super().__init__(remote, schema, **kwargs)
 
     @classmethod
     def from_server(cls, connection_string, helper):
@@ -681,23 +810,18 @@ class BaseOvnIdl(Ml2OvnIdlBase):
 class BaseOvnSbIdl(Ml2OvnIdlBase):
     @classmethod
     def from_server(cls, connection_string, helper):
-        if 'Chassis_Private' in helper.schema_json['tables']:
-            helper.register_table('Chassis_Private')
+        helper.register_table('Chassis_Private')
         helper.register_table('Chassis')
         helper.register_table('Encap')
         helper.register_table('Port_Binding')
         helper.register_table('Datapath_Binding')
-        try:
-            return cls(connection_string, helper, leader_only=False)
-        except TypeError:
-            # TODO(twilson) We can remove this when we require ovs>=2.12.0
-            return cls(connection_string, helper)
+        return cls(connection_string, helper, leader_only=False)
 
 
 class OvnIdl(BaseOvnIdl):
 
     def __init__(self, driver, remote, schema, **kwargs):
-        super(OvnIdl, self).__init__(remote, schema, **kwargs)
+        super().__init__(remote, schema, **kwargs)
         self.driver = driver
         self.notify_handler = OvnDbNotifyHandler(driver)
 
@@ -716,41 +840,19 @@ class OvnIdl(BaseOvnIdl):
 class OvnIdlDistributedLock(BaseOvnIdl):
 
     def __init__(self, driver, remote, schema, **kwargs):
-        super(OvnIdlDistributedLock, self).__init__(remote, schema, **kwargs)
+        super().__init__(remote, schema, **kwargs)
         self.driver = driver
         self.notify_handler = OvnDbNotifyHandler(driver)
         self._node_uuid = self.driver.node_uuid
         self._hash_ring = hash_ring_manager.HashRingManager(
             self.driver.hash_ring_group)
-        self._last_touch = None
-        # This is a map of tables that may be new after OVN database is updated
-        self._tables_to_register = {
-            'OVN_Southbound': ['Chassis_Private'],
-        }
-
-    def handle_db_schema_changes(self, event, row):
-        if (event == row_event.RowEvent.ROW_CREATE and
-                row._table.name == 'Database'):
-            try:
-                tables = self._tables_to_register[row.name]
-            except KeyError:
-                return
-
-            self.update_tables(tables, row.schema[0])
-
-            if self.driver.agent_chassis_table == 'Chassis_Private':
-                if 'Chassis_Private' not in self.tables:
-                    self.driver.agent_chassis_table = 'Chassis'
-            else:
-                if 'Chassis_Private' in self.tables:
-                    self.driver.agent_chassis_table = 'Chassis_Private'
 
     def notify(self, event, row, updates=None):
         try:
-            self.handle_db_schema_changes(event, row)
             self.notify_handler.notify(event, row, updates, global_=True)
             try:
-                target_node = self._hash_ring.get_node(str(row.uuid))
+                target_node, node_last_touch = self._hash_ring.get_node(
+                    str(row.uuid))
             except exceptions.HashRingIsEmpty as e:
                 LOG.error('HashRing is empty, error: %s', e)
                 return
@@ -759,17 +861,18 @@ class OvnIdlDistributedLock(BaseOvnIdl):
 
             # If the worker hasn't been health checked by the maintenance
             # thread (see bug #1834498), indicate that it's alive here
-            time_now = timeutils.utcnow()
-            touch_timeout = time_now - datetime.timedelta(
+            touch_timeout = timeutils.utcnow() - datetime.timedelta(
                 seconds=ovn_const.HASH_RING_TOUCH_INTERVAL)
-            if not self._last_touch or touch_timeout >= self._last_touch:
+            if not node_last_touch or touch_timeout >= node_last_touch:
                 # NOTE(lucasagomes): Guard the db operation with an exception
                 # handler. If heartbeating fails for whatever reason, log
                 # the error and continue with processing the event
                 try:
                     ctx = neutron_context.get_admin_context()
+                    LOG.debug(
+                        'Touching Hash Ring node "%s" from IDL notify handler',
+                        self._node_uuid)
                     ovn_hash_ring_db.touch_node(ctx, self._node_uuid)
-                    self._last_touch = time_now
                 except Exception:
                     LOG.exception('Hash Ring node %s failed to heartbeat',
                                   self._node_uuid)
@@ -792,16 +895,22 @@ class OvnIdlDistributedLock(BaseOvnIdl):
 class OvnNbIdl(OvnIdlDistributedLock):
 
     def __init__(self, driver, remote, schema):
-        super(OvnNbIdl, self).__init__(driver, remote, schema)
+        super().__init__(driver, remote, schema)
         self._lsp_update_up_event = LogicalSwitchPortUpdateUpEvent(driver)
         self._lsp_update_down_event = LogicalSwitchPortUpdateDownEvent(driver)
         self._lsp_create_event = LogicalSwitchPortCreateEvent(driver)
+        self._lsp_lrp_event = (
+            LogicalSwitchPortUpdateLogicalRouterPortEvent(driver))
         self._fip_create_delete_event = FIPAddDeleteEvent(driver)
+        self._ha_chassis_group_event = HAChassisGroupRouterEvent(driver)
 
         self.notify_handler.watch_events([self._lsp_create_event,
                                           self._lsp_update_up_event,
                                           self._lsp_update_down_event,
-                                          self._fip_create_delete_event])
+                                          self._fip_create_delete_event,
+                                          self._lsp_lrp_event,
+                                          self._ha_chassis_group_event,
+                                          ])
 
     @classmethod
     def from_server(cls, connection_string, helper, driver):
@@ -809,52 +918,32 @@ class OvnNbIdl(OvnIdlDistributedLock):
         helper.register_all()
         return cls(driver, connection_string, helper)
 
-    def unwatch_logical_switch_port_create_events(self):
-        """Unwatch the logical switch port create events.
-
-        When the ovs idl client connects to the ovsdb-server, it gets
-        a dump of all logical switch ports as events and we need to process
-        them at start up.
-        After the startup, there is no need to watch these events.
-        So unwatch these events.
-        """
-        self.notify_handler.unwatch_events([self._lsp_create_event])
-        del self._lsp_create_event
-
-    def post_connect(self):
-        self.unwatch_logical_switch_port_create_events()
-
 
 class OvnSbIdl(OvnIdlDistributedLock):
 
     def __init__(self, driver, remote, schema, **kwargs):
-        super(OvnSbIdl, self).__init__(driver, remote, schema, **kwargs)
+        super().__init__(driver, remote, schema, **kwargs)
         self.notify_handler.watch_events([
             ChassisAgentDeleteEvent(self.driver),
             ChassisAgentDownEvent(self.driver),
             ChassisAgentWriteEvent(self.driver),
             ChassisAgentTypeChangeEvent(self.driver),
-            ChassisMetadataAgentWriteEvent(self.driver),
+            ChassisOVNAgentWriteEvent(self.driver),
             PortBindingUpdateVirtualPortsEvent(driver),
             placement.ChassisBandwidthConfigEvent(driver),
         ])
 
     @classmethod
     def from_server(cls, connection_string, helper, driver):
-        if 'Chassis_Private' in helper.schema_json['tables']:
-            helper.register_table('Chassis_Private')
         if 'FDB' in helper.schema_json['tables']:
             helper.register_table('FDB')
+        helper.register_table('Chassis_Private')
         helper.register_table('Chassis')
         helper.register_table('Encap')
         helper.register_table('Port_Binding')
         helper.register_table('Datapath_Binding')
         helper.register_columns('SB_Global', ['external_ids'])
-        try:
-            return cls(driver, connection_string, helper, leader_only=False)
-        except TypeError:
-            # TODO(twilson) We can remove this when we require ovs>=2.12.0
-            return cls(driver, connection_string, helper)
+        return cls(driver, connection_string, helper, leader_only=False)
 
     def post_connect(self):
         """Watch Chassis events.
