@@ -28,7 +28,6 @@ from neutron_lib.plugins import directory
 from neutron_lib.services import constants as service_const
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_policy import opts
 from oslo_policy import policy
 from oslo_utils import excutils
 import stevedore
@@ -51,14 +50,6 @@ _RESOURCE_FOREIGN_KEYS = {
     'security_groups': 'security_group_id'
 }
 
-# TODO(slaweq): Remove overriding the default value of config options
-# 'policy_file' once oslo_policy change its default value to what
-# is overridden here.
-DEFAULT_POLICY_FILE = 'policy.yaml'
-opts.set_defaults(
-    cfg.CONF,
-    DEFAULT_POLICY_FILE)
-
 
 def reset():
     global _ENFORCER
@@ -107,7 +98,10 @@ def get_resource_and_action(action, pluralized=None):
        resource and action extracted from api operation.
     """
     data = action.split(':', 1)[0].split('_', 1)
-    resource = pluralized or ("%ss" % data[-1])
+    if "tags" in data[-1]:
+        resource = data[-1].replace("_tags", "")
+    else:
+        resource = pluralized or ("%ss" % data[-1])
     enforce_attr_based_check = data[0] not in ('get', 'delete')
     return (resource, enforce_attr_based_check)
 
@@ -144,8 +138,9 @@ def _should_validate_sub_attributes(attribute, sub_attr):
     """Verify that sub-attributes are iterable and should be validated."""
     validate = attribute.get('validate')
     return (validate and isinstance(sub_attr, abc.Iterable) and
-            any(k.startswith('type:dict') and v
-                for (k, v) in validate.items()))
+            any((k.startswith('type:dict') or
+                 k.startswith('type:list_of_dict')) and
+                v for (k, v) in validate.items()))
 
 
 def _build_subattr_match_rule(attr_name, attr, action, target):
@@ -154,7 +149,8 @@ def _build_subattr_match_rule(attr_name, attr, action, target):
     # typing for API attributes
     # Expect a dict as type descriptor
     validate = attr['validate']
-    key = [k for k in validate.keys() if k.startswith('type:dict')]
+    key = [k for k in validate.keys() if (k.startswith('type:dict') or
+                                          k.startswith('type:list_of_dict'))]
     if not key:
         LOG.warning("Unable to find data type descriptor for attribute %s",
                     attr_name)
@@ -165,11 +161,18 @@ def _build_subattr_match_rule(attr_name, attr, action, target):
                   "generate any sub-attr policy rule for %s.",
                   attr_name)
         return
+
+    if key[0].startswith('type:list_of_dict'):
+        target_attributes = set()
+        for _attrs in target[attr_name]:
+            target_attributes = target_attributes.union(set(_attrs.keys()))
+    else:
+        target_attributes = target[attr_name]
     sub_attr_rules = [policy.RuleCheck('rule', '%s:%s:%s' %
                                        (action, attr_name,
                                         sub_attr_name)) for
                       sub_attr_name in data if sub_attr_name in
-                      target[attr_name]]
+                      target_attributes]
     return policy.AndCheck(sub_attr_rules)
 
 
@@ -179,7 +182,7 @@ def _build_list_of_subattrs_rule(attr_name, attribute_value, action):
         if isinstance(sub_attr, dict):
             for k in sub_attr:
                 rules.append(policy.RuleCheck(
-                    'rule', '%s:%s:%s' % (action, attr_name, k)))
+                    'rule', f'{action}:{attr_name}:{k}'))
     if rules:
         return policy.AndCheck(rules)
 
@@ -224,7 +227,7 @@ def _build_match_rule(action, target, pluralized):
                     attribute = res_map[resource][attribute_name]
                     if 'enforce_policy' in attribute:
                         attr_rule = policy.RuleCheck(
-                            'rule', '%s:%s' % (action, attribute_name))
+                            'rule', f'{action}:{attribute_name}')
                         # Build match entries for sub-attributes
                         if _should_validate_sub_attributes(
                                 attribute, target[attribute_name]):
@@ -274,10 +277,10 @@ class OwnerCheck(policy.Check):
                           match)
             LOG.exception(err_reason)
             raise exceptions.PolicyInitError(
-                policy="%s:%s" % (kind, match),
+                policy=f"{kind}:{match}",
                 reason=err_reason)
         self._cache = cache._get_memory_cache_region(expiration_time=5)
-        super(OwnerCheck, self).__init__(kind, match)
+        super().__init__(kind, match)
 
     # NOTE(slaweq): It seems we need to have it like that, otherwise we hit
     # TypeError: cannot pickle '_thread.RLock' object
@@ -346,13 +349,14 @@ class OwnerCheck(policy.Check):
                               self.target_field)
                 LOG.error(err_reason)
                 raise exceptions.PolicyCheckError(
-                    policy="%s:%s" % (self.kind, self.match),
+                    policy=f"{self.kind}:{self.match}",
                     reason=err_reason)
             parent_foreign_key = _RESOURCE_FOREIGN_KEYS.get(
                 "%ss" % parent_res, None)
             if parent_res == constants.EXT_PARENT_PREFIX:
                 for resource in service_const.EXT_PARENT_RESOURCE_MAPPING:
-                    key = "%s_%s_id" % (constants.EXT_PARENT_PREFIX, resource)
+                    key = "{}_{}_id".format(
+                        constants.EXT_PARENT_PREFIX, resource)
                     if key in target:
                         parent_foreign_key = key
                         parent_res = resource
@@ -363,7 +367,7 @@ class OwnerCheck(policy.Check):
                               {'match': self.match, 'res': parent_res})
                 LOG.error(err_reason)
                 raise exceptions.PolicyCheckError(
-                    policy="%s:%s" % (self.kind, self.match),
+                    policy=f"{self.kind}:{self.match}",
                     reason=err_reason)
 
             target_copy[self.target_field] = self._extract(
@@ -385,8 +389,8 @@ class FieldCheck(policy.Check):
         resource, field_value = match.split(':', 1)
         field, value = field_value.split('=', 1)
 
-        super(FieldCheck, self).__init__(kind, '%s:%s:%s' %
-                                         (resource, field, value))
+        super().__init__(kind, '%s:%s:%s' %
+                         (resource, field, value))
 
         # Value might need conversion - we need help from the attribute map
 
@@ -433,7 +437,7 @@ class FieldCheck(policy.Check):
                           {'field': self.field, 'target_dict': target_dict})
                 return
             project_id = target_dict.get('project_id')
-            ctx = (context.Context(tenant_id=project_id) if project_id
+            ctx = (context.Context(project_id=project_id) if project_id
                    else context.get_admin_context())
             plugin = directory.get_plugin()
             network = plugin.get_network(ctx, target_network_id)
@@ -460,8 +464,7 @@ def log_rule_list(match_rule):
         LOG.debug("Enforcing rules: %s", rules)
 
 
-def check(context, action, target, plugin=None, might_not_exist=False,
-          pluralized=None):
+def check(context, action, target, might_not_exist=False, pluralized=None):
     """Verifies that the action is valid on the target in this context.
 
     :param context: neutron context
@@ -470,8 +473,6 @@ def check(context, action, target, plugin=None, might_not_exist=False,
     :param target: dictionary representing the object of the action
         for object creation this should be a dictionary representing the
         location of the object e.g. ``{'project_id': context.project_id}``
-    :param plugin: currently unused and deprecated.
-        Kept for backward compatibility.
     :param might_not_exist: If True the policy check is skipped (and the
         function returns True) if the specified policy does not exist.
         Defaults to false.
@@ -500,7 +501,7 @@ def check(context, action, target, plugin=None, might_not_exist=False,
                              pluralized=pluralized)
 
 
-def enforce(context, action, target, plugin=None, pluralized=None):
+def enforce(context, action, target, pluralized=None):
     """Verifies that the action is valid on the target in this context.
 
     :param context: neutron context
@@ -509,8 +510,6 @@ def enforce(context, action, target, plugin=None, pluralized=None):
     :param target: dictionary representing the object of the action
         for object creation this should be a dictionary representing the
         location of the object e.g. ``{'project_id': context.project_id}``
-    :param plugin: currently unused and deprecated.
-        Kept for backward compatibility.
     :param pluralized: pluralized case of resource
         e.g. firewall_policy -> pluralized = "firewall_policies"
 
