@@ -13,16 +13,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api.definitions import flavors
+from neutron_lib.api.definitions import servicetype
+from neutron_lib.plugins import constants
+from neutron_lib.services import base as service_base
+
 from neutron.db import flavors_db
-from neutron.plugins.common import constants
-from neutron.services import service_base
 
 
 class FlavorsPlugin(service_base.ServicePluginBase,
                     flavors_db.FlavorsDbMixin):
     """Implements Neutron Flavors Service plugin."""
 
-    supported_extension_aliases = ['flavors', 'service-type']
+    supported_extension_aliases = [flavors.ALIAS, servicetype.ALIAS]
+
+    __filter_validation_support = True
 
     @classmethod
     def get_plugin_type(cls):
