@@ -15,7 +15,7 @@
 from neutron._i18n import _
 
 
-class _FeatureFlag(object):
+class _FeatureFlag:
 
     def is_compatible(self, value):
         if value == self.requires:
@@ -31,12 +31,13 @@ class _FeatureFlag(object):
             raise RuntimeError(_("A driver can't require a feature and not "
                                  "support it."))
 
+
 UNSUPPORTED = _FeatureFlag(supports=False, requires=False)
 OPTIONAL = _FeatureFlag(supports=True, requires=False)
 MANDATORY = _FeatureFlag(supports=True, requires=True)
 
 
-class L3ServiceProvider(object):
+class L3ServiceProvider:
     """Base class for L3 service provider drivers.
 
     On __init__ this will be given a handle to the l3 plugin. It is then the
@@ -58,3 +59,10 @@ class L3ServiceProvider(object):
 
     def __init__(self, l3plugin):
         self.l3plugin = l3plugin
+
+    def owns_router(self, context, router_id):
+        """Returns True if router is associated with driver, else False."""
+        if not router_id:
+            return False
+        return self.l3plugin.l3_driver_controller.get_provider_for_router(
+            context, router_id) == self
