@@ -13,6 +13,7 @@
 # under the License.
 
 import ipaddress
+
 import netaddr
 
 from neutron_lib.agent import topics
@@ -41,7 +42,7 @@ class MeteringPlugin(metering_db.MeteringDbMixin):
     __filter_validation_support = True
 
     def __init__(self):
-        super(MeteringPlugin, self).__init__()
+        super().__init__()
 
         self.meter_rpc = metering_rpc_agent_api.MeteringAgentNotifyAPI()
         rpc_worker = service.RpcWorker([self], worker_process_count=0)
@@ -56,7 +57,7 @@ class MeteringPlugin(metering_db.MeteringDbMixin):
         return self.conn.consume_in_threads()
 
     def create_metering_label(self, context, metering_label):
-        label = super(MeteringPlugin, self).create_metering_label(
+        label = super().create_metering_label(
             context, metering_label)
 
         data = self.get_sync_data_metering(context)
@@ -66,19 +67,17 @@ class MeteringPlugin(metering_db.MeteringDbMixin):
 
     def delete_metering_label(self, context, label_id):
         data = self.get_sync_data_metering(context, label_id)
-        label = super(MeteringPlugin, self).delete_metering_label(
+        super().delete_metering_label(
             context, label_id)
 
         self.meter_rpc.remove_metering_label(context, data)
 
-        return label
-
     def create_metering_label_rule(self, context, metering_label_rule):
         metering_label_rule = metering_label_rule['metering_label_rule']
         MeteringPlugin.validate_metering_label_rule(metering_label_rule)
         self.check_for_rule_overlaps(context, metering_label_rule)
 
-        rule = super(MeteringPlugin, self).create_metering_label_rule(
+        rule = super().create_metering_label_rule(
             context, metering_label_rule)
 
         if rule.get("remote_ip_prefix"):
@@ -104,16 +103,16 @@ class MeteringPlugin(metering_db.MeteringDbMixin):
             metering_label_rule, "destination_ip_prefix")
 
         if metering_label_rule.get("remote_ip_prefix"):
-            if metering_label_rule.get("source_ip_prefix") or \
-                    metering_label_rule.get("destination_ip_prefix"):
+            if (metering_label_rule.get("source_ip_prefix") or
+                    metering_label_rule.get("destination_ip_prefix")):
                 raise neutron_exc.Invalid(
                     "Cannot use 'remote-ip-prefix' in conjunction "
                     "with 'source-ip-prefix' or 'destination-ip-prefix'.")
 
-        none_ip_prefix_informed = not metering_label_rule.get(
-            'remote_ip_prefix') and not metering_label_rule.get(
-            'source_ip_prefix') and not metering_label_rule.get(
-            'destination_ip_prefix')
+        none_ip_prefix_informed = (
+            not metering_label_rule.get('remote_ip_prefix') and
+            not metering_label_rule.get('source_ip_prefix') and
+            not metering_label_rule.get('destination_ip_prefix'))
 
         if none_ip_prefix_informed:
             raise neutron_exc.Invalid(
@@ -169,7 +168,7 @@ class MeteringPlugin(metering_db.MeteringDbMixin):
                     remote_ip_prefix=remote_ip_prefix)
 
     def delete_metering_label_rule(self, context, rule_id):
-        rule = super(MeteringPlugin, self).delete_metering_label_rule(
+        rule = super().delete_metering_label_rule(
             context, rule_id)
 
         data = self.get_sync_data_for_rule(context, rule)
