@@ -13,7 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import signal
 
 from oslo_config import cfg
 
@@ -23,19 +22,22 @@ from neutron.agent.linux import ip_lib
 from neutron.agent.linux import keepalived
 from neutron.common import utils as common_utils
 from neutron.conf.agent.l3 import config as l3_config
+from neutron.conf.agent.l3 import ha as ha_config
 from neutron.tests.common import net_helpers
 from neutron.tests.functional.agent.linux import helpers
 from neutron.tests.functional import base
 from neutron.tests.unit.agent.linux import test_keepalived
+from neutron_lib.exceptions import ProcessExecutionError
 
 
 class KeepalivedManagerTestCase(base.BaseSudoTestCase,
                                 test_keepalived.KeepalivedConfBaseMixin):
 
     def setUp(self):
-        super(KeepalivedManagerTestCase, self).setUp()
+        super().setUp()
         l3_config.register_l3_agent_config_opts(l3_config.OPTS, cfg.CONF)
-        cfg.CONF.set_override('check_child_processes_interval', 1, 'AGENT')
+        ha_config.register_l3_agent_ha_opts()
+        self.config(check_child_processes_interval=1, group='AGENT')
 
         self.expected_config = self._get_config()
         self.process_monitor = external_process.ProcessMonitor(cfg.CONF,
@@ -51,12 +53,17 @@ class KeepalivedManagerTestCase(base.BaseSudoTestCase,
         self.addCleanup(self._stop_keepalived_manager)
 
     def _stop_keepalived_manager(self):
-        self.manager.disable()
         try:
-            common_utils.wait_until_true(
-                lambda: not self.manager.get_process().active, timeout=5)
-        except common_utils.WaitTimeout:
-            self.manager.get_process().disable(sig=signal.SIGKILL)
+            self.manager.disable()
+        except ProcessExecutionError as process_err:
+            # self.manager.disable() will perform SIGTERM->wait->SIGKILL
+            # (if needed) on the process. However, it is sometimes possible
+            # that SIGKILL gets called on a process that just exited due to
+            # SIGTERM. Ignore this condition so the test is not marked as
+            # failed.
+            if not (len(process_err.args) > 0 and
+                    "No such process" in process_err.args[0]):
+                raise
 
     def _prepare_devices(self):
         # NOTE(slaweq): those are devices used in keepalived config file,
@@ -92,7 +99,8 @@ class KeepalivedManagerTestCase(base.BaseSudoTestCase,
 
         # Exit the process, and see that when it comes back
         # It's indeed a different process
-        self.ip_wrapper.netns.execute(['kill', exit_code, pid])
+        self.ip_wrapper.netns.execute(['kill', exit_code, pid],
+                                      privsep_exec=True)
         common_utils.wait_until_true(
             lambda: process.active and pid != process.pid,
             timeout=5,
@@ -119,11 +127,13 @@ class KeepalivedManagerTestCase(base.BaseSudoTestCase,
         self._spawn_keepalived(self.manager)
 
     def test_keepalived_spawns_conflicting_pid_base_process(self):
+        self.skipTest('bug 1921154')
         process = self.manager.get_process()
         pid_file = process.get_pid_file_name()
         self._test_keepalived_spawns_conflicting_pid(process, pid_file)
 
     def test_keepalived_spawns_conflicting_pid_vrrp_subprocess(self):
+        self.skipTest('bug 1921154')
         process = self.manager.get_process()
         pid_file = process.get_pid_file_name()
         self._test_keepalived_spawns_conflicting_pid(
