@@ -27,7 +27,7 @@ import testtools
 from neutron.agent.linux import iptables_manager
 from neutron.agent.ovn.metadata import agent
 from neutron.agent.ovn.metadata import ovsdb
-from neutron.agent.ovn.metadata import server as metadata_server
+from neutron.agent.ovn.metadata import server_socket as metadata_server
 from neutron.common.ovn import constants as ovn_const
 from neutron.common import utils as n_utils
 from neutron.conf.agent.metadata import config as meta_config
@@ -50,29 +50,25 @@ class MetadataAgentHealthEvent(event.WaitEvent):
     def __init__(self, chassis, sb_cfg, timeout=5):
         self.chassis = chassis
         self.sb_cfg = sb_cfg
-        super(MetadataAgentHealthEvent, self).__init__(
+        super().__init__(
             (self.ROW_UPDATE,),
             AGENT_CHASSIS_TABLE,
             (('name', '=', self.chassis),),
             timeout=timeout)
 
     def matches(self, event, row, old=None):
-        if not super(MetadataAgentHealthEvent, self).matches(event, row, old):
+        if not super().matches(event, row, old):
             return False
         return int(row.external_ids.get(
             ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY, 0)) >= self.sb_cfg
 
 
-class MetadataPortCreateEvent(event.WaitEvent):
-    event_name = 'MetadataPortCreateEvent'
-
-    def __init__(self, metadata_port, timeout=5):
+class PortBindingUpdateEvent(event.WaitEvent):
+    def __init__(self, lsp, timeout=5):
         table = 'Port_Binding'
-        events = (self.ROW_CREATE,)
-        conditions = (('logical_port', '=', metadata_port),)
-        super(MetadataPortCreateEvent, self).__init__(
-            events, table, conditions, timeout=timeout
-        )
+        events = (self.ROW_UPDATE,)
+        conditions = (('logical_port', '=', lsp),)
+        super().__init__(events, table, conditions, timeout=timeout)
 
 
 class TestMetadataAgent(base.TestOVNFunctionalBase):
@@ -80,7 +76,7 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
     FAKE_CHASSIS_HOST = 'ovn-host-fake'
 
     def setUp(self):
-        super(TestMetadataAgent, self).setUp()
+        super().setUp()
         self.handler = self.sb_api.idl.notify_handler
         # We only have OVN NB and OVN SB running for functional tests
         self.mock_ovsdb_idl = mock.Mock()
@@ -104,7 +100,7 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
         meta_config_ovn.setup_privsep()
 
         ovn_sb_db = self.ovsdb_server_mgr.get_ovsdb_connection_path('sb')
-        conf.set_override('ovn_sb_connection', ovn_sb_db, group='ovn')
+        conf.set_override('ovn_sb_connection', [ovn_sb_db], group='ovn')
         conf.set_override('metadata_workers', '0')
 
         self.chassis_name = self.add_fake_chassis(self.FAKE_CHASSIS_HOST)
@@ -120,6 +116,9 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
                 check_error=True)
             self.assertEqual(external_ids[ovn_const.OVN_AGENT_OVN_BRIDGE],
                              self.OVN_BRIDGE)
+            self.assertEqual(
+                external_ids[ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY],
+                '0')
 
         # Metadata agent will open connections to OVS and SB databases.
         # Close connections to them when the test ends,
@@ -129,16 +128,6 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
         return agt
 
     def test_metadata_agent_healthcheck(self):
-        chassis_row = self.sb_api.db_find(
-            AGENT_CHASSIS_TABLE,
-            ('name', '=', self.chassis_name)).execute(
-            check_error=True)[0]
-
-        # Assert that, prior to creating a resource the metadata agent
-        # didn't populate the external_ids from the Chassis
-        self.assertNotIn(ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY,
-                         chassis_row['external_ids'])
-
         # Let's list the agents to force the nb_cfg to be bumped on NB
         # db, which will automatically increment the nb_cfg counter on
         # NB_Global and make ovn-controller copy it over to SB_Global. Upon
@@ -453,7 +442,7 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
 
     def test_metadata_proxy_handler_idl(self):
         # This test relies on the configuration option metadata_workers=0
-        proxy_sb_idl = self.agent._proxy.server._server._application.sb_idl
+        proxy_sb_idl = metadata_server.MetadataProxyHandler._sb_idl
         agent_sb_idl = self.agent.sb_idl
         self.assertEqual(agent_sb_idl, proxy_sb_idl)
 
@@ -477,7 +466,8 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
                 agent.MetadataAgent, 'provision_datapath') as m_provision:
 
             # Update the additional_chassis
-            self.sb_api.db_set('Port_Binding', pb.uuid,
+            self.sb_api.db_set(
+                'Port_Binding', pb.uuid,
                 additional_chassis=[agent_chassis.uuid]).execute(
                     check_error=True, log_errors=True)
 
@@ -500,9 +490,12 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
 
         lswitchport_name, lswitch_name = self._create_logical_switch_port()
 
+        pb_event = PortBindingUpdateEvent(lswitchport_name)
+        self.agent.sb_idl.idl.notify_handler.watch_event(pb_event)
         self.sb_api.lsp_bind(
             lswitchport_name, agent_chassis.name).execute(
                 check_error=True, log_errors=True)
+        self.assertTrue(pb_event.wait())
         pb = idlutils.row_by_value(
             self.sb_api, 'Port_Binding', 'logical_port', lswitchport_name)
 
@@ -512,7 +505,8 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
             # Update the additional_chassis, the agent should not see the
             # notification because it has only its own chassis row locally and
             # does not see other chassis
-            self.sb_api.db_set('Port_Binding', pb.uuid,
+            self.sb_api.db_set(
+                'Port_Binding', pb.uuid,
                 additional_chassis=[other_chassis.uuid]).execute(
                     check_error=True, log_errors=True)
 
@@ -543,7 +537,8 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
                 agent.MetadataAgent, 'provision_datapath') as m_provision:
 
             # Update the additional_chassis
-            self.sb_api.db_set('Port_Binding', pb.uuid,
+            self.sb_api.db_set(
+                'Port_Binding', pb.uuid,
                 additional_chassis=[agent_chassis.uuid]).execute(
                     check_error=True, log_errors=True)
 
@@ -558,7 +553,7 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
             # Remove the additional_chassis but keep the chassis. This is
             # simulates the live migration has failed
             self.sb_api.db_set('Port_Binding', pb.uuid,
-                additional_chassis=[]).execute(
+                               additional_chassis=[]).execute(
                     check_error=True, log_errors=True)
 
             n_utils.wait_until_true(
@@ -587,7 +582,8 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
                 agent.MetadataAgent, 'provision_datapath') as m_provision:
 
             # Update the additional_chassis
-            self.sb_api.db_set('Port_Binding', pb.uuid,
+            self.sb_api.db_set(
+                'Port_Binding', pb.uuid,
                 additional_chassis=[agent_chassis.uuid]).execute(
                     check_error=True, log_errors=True)
 
@@ -599,7 +595,8 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
 
             m_provision.reset_mock()
 
-            self.sb_api.db_set('Port_Binding', pb.uuid,
+            self.sb_api.db_set(
+                'Port_Binding', pb.uuid,
                 additional_chassis=[], chassis=agent_chassis.uuid).execute(
                     check_error=True, log_errors=True)
 
@@ -629,7 +626,8 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
                 agent.MetadataAgent, 'provision_datapath') as m_provision:
 
             # Update the additional_chassis
-            self.sb_api.db_set('Port_Binding', pb.uuid,
+            self.sb_api.db_set(
+                'Port_Binding', pb.uuid,
                 additional_chassis=[agent_chassis.uuid]).execute(
                     check_error=True, log_errors=True)
 
@@ -641,7 +639,8 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
 
             m_provision.reset_mock()
 
-            self.sb_api.db_set('Port_Binding', pb.uuid,
+            self.sb_api.db_set(
+                'Port_Binding', pb.uuid,
                 additional_chassis=[], chassis=new_chassis_uuid).execute(
                     check_error=True, log_errors=True)
 
@@ -670,9 +669,12 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
 
         lswitchport_name, lswitch_name = self._create_logical_switch_port()
 
+        pb_event = PortBindingUpdateEvent(lswitchport_name)
+        self.agent.sb_idl.idl.notify_handler.watch_event(pb_event)
         self.sb_api.lsp_bind(
             lswitchport_name, agent_chassis.name).execute(
                 check_error=True, log_errors=True)
+        self.assertTrue(pb_event.wait())
         pb = idlutils.row_by_value(
             self.sb_api, 'Port_Binding', 'logical_port', lswitchport_name)
 
@@ -680,7 +682,7 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
                 agent.MetadataAgent, 'provision_datapath') as m_provision:
 
             self.sb_api.db_add('Port_Binding', pb.uuid,
-                'external_ids', {'foo': 'bar'}).execute(
+                               'external_ids', {'foo': 'bar'}).execute(
                     check_error=True, log_errors=True)
 
             with testtools.ExpectedException(NoDatapathProvision):
@@ -689,3 +691,26 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
                     timeout=1,
                     exception=NoDatapathProvision(
                         "Provisioning wasn't triggered"))
+
+    def test__cleanup_previous_tags(self):
+        external_ids = {
+            ovn_const.OVN_AGENT_NEUTRON_SB_CFG_KEY: '1',
+            ovn_const.OVN_AGENT_NEUTRON_DESC_KEY: 'description',
+            ovn_const.OVN_AGENT_NEUTRON_ID_KEY: uuidutils.generate_uuid()}
+        self.sb_api.db_set(
+            'Chassis_Private', self.chassis_name,
+            ('external_ids', external_ids)).execute(check_error=True)
+
+        self.agent._cleanup_previous_tags()
+        external_ids = self.sb_api.db_get(
+            'Chassis_Private', self.chassis_name,
+            'external_ids').execute(check_error=True)
+        for _key in (ovn_const.OVN_AGENT_NEUTRON_SB_CFG_KEY,
+                     ovn_const.OVN_AGENT_NEUTRON_DESC_KEY,
+                     ovn_const.OVN_AGENT_NEUTRON_ID_KEY):
+            self.assertNotIn(_key, external_ids)
+
+        # Just in case, check that we are NOT deleting the needed tags.
+        for _key in (ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY,
+                     ovn_const.OVN_AGENT_METADATA_ID_KEY):
+            self.assertIn(_key, external_ids)
