@@ -24,15 +24,24 @@ from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
+from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as ovn_config
+from neutron.db import ovn_revision_numbers_db as rev_db
 from neutron.tests.functional import base
 
 
 class TestNBDbResources(base.TestOVNFunctionalBase):
     _extension_drivers = ['dns']
 
+    def _is_nb_global_ready(self):
+        try:
+            next(iter(self.nb_api.tables['NB_Global'].rows))
+        except StopIteration:
+            return False
+        return True
+
     def setUp(self):
-        super(TestNBDbResources, self).setUp()
+        super().setUp()
         self.orig_get_random_mac = n_net.get_random_mac
         cfg.CONF.set_override('quota_subnet', -1, group='QUOTAS')
         ovn_config.cfg.CONF.set_override('ovn_metadata_enabled',
@@ -40,6 +49,12 @@ class TestNBDbResources(base.TestOVNFunctionalBase):
                                          group='ovn')
         ovn_config.cfg.CONF.set_override('dns_domain', 'ovn.test')
 
+        # Wait for NB_Global table, for details see: LP #1956965
+        n_utils.wait_until_true(
+            self._is_nb_global_ready,
+            timeout=15, sleep=1
+        )
+
     # FIXME(lucasagomes): Map the revision numbers properly instead
     # of stripping them out. Currently, tests like test_dhcp_options()
     # are quite complex making it difficult to map the exact the revision
@@ -58,7 +73,7 @@ class TestNBDbResources(base.TestOVNFunctionalBase):
                 'cidr': row.cidr, 'external_ids': ext_ids,
                 'options': row.options})
 
-        self.assertItemsEqual(expected_dhcp_options_rows,
+        self.assertCountEqual(expected_dhcp_options_rows,
                               observed_dhcp_options_rows)
 
     def _verify_dhcp_option_row_for_port(self, port_id,
@@ -98,7 +113,7 @@ class TestNBDbResources(base.TestOVNFunctionalBase):
 
     def _get_subnet_dhcp_mac(self, subnet):
         mac_key = 'server_id' if subnet['ip_version'] == 6 else 'server_mac'
-        dhcp_options = self.mech_driver._nb_ovn.get_subnet_dhcp_options(
+        dhcp_options = self.mech_driver.nb_ovn.get_subnet_dhcp_options(
             subnet['id'])['subnet']
         return dhcp_options.get('options', {}).get(
             mac_key) if dhcp_options else None
@@ -362,7 +377,7 @@ class TestNBDbResources(base.TestOVNFunctionalBase):
                         'mtu': str(n1['network']['mtu']),
                         'router': subnet['gateway_ip'],
                         'ip_forward_enable': '1',
-                        'tftp_server': '10.0.0.100',
+                        'tftp_server': '"10.0.0.100"',
                         'domain_name': '"%s"' % cfg.CONF.dns_domain,
                         'dns_server': '20.20.20.20'}}
         expected_dhcp_v4_options_rows['v4-' + p2['port']['id']] = \
@@ -426,7 +441,7 @@ class TestNBDbResources(base.TestOVNFunctionalBase):
                         'dns_server': '{10.10.10.10}',
                         'mtu': str(n1['network']['mtu']),
                         'router': subnet['gateway_ip'],
-                        'tftp_server': '100.0.0.100'}}
+                        'tftp_server': '"100.0.0.100"'}}
         expected_dhcp_v4_options_rows['v4-' + p4['port']['id']] = \
             expected_dhcp_options_rows['v4-' + p4['port']['id']]
         expected_dhcp_v6_options_rows['v6-' + p4['port']['id']] = \
@@ -694,7 +709,7 @@ class TestNBDbResources(base.TestOVNFunctionalBase):
                         'dns_server': '{10.10.10.10}',
                         'mtu': '1200',
                         'router': subnet['gateway_ip'],
-                        'tftp_server': '8.8.8.8'}}
+                        'tftp_server': '"8.8.8.8"'}}
         self._verify_dhcp_option_rows(expected_dhcp_options_rows)
         self._verify_dhcp_option_row_for_port(
             p1['id'], expected_dhcp_options_rows['v4-' + p1['id']],
@@ -760,13 +775,14 @@ class TestNBDbResources(base.TestOVNFunctionalBase):
         # is not configured as domain_name.
         # Parameter taken from configuration
         # should be set instead.
+        mtu = str(1480 - cfg.CONF.ml2_type_geneve.max_header_size)
         expected_dhcp_options_rows = {
             'cidr': '10.0.0.0/24',
             'external_ids': {'subnet_id': subnet['id']},
             'options': {'dns_server': '{10.10.10.10}',
                         'domain_name': '"%s"' % cfg.CONF.dns_domain,
                         'lease_time': '43200',
-                        'mtu': '1450',
+                        'mtu': mtu,
                         'router': '10.0.0.1',
                         'server_id': '10.0.0.1',
                         'server_mac': dhcp_mac}}
@@ -781,13 +797,14 @@ class TestNBDbResources(base.TestOVNFunctionalBase):
         p = self._make_port(self.fmt, n1['network']['id'],
                             fixed_ips=[{'subnet_id': subnet['id']}])
         dhcp_mac = self._get_subnet_dhcp_mac(subnet)
+        mtu = str(1480 - cfg.CONF.ml2_type_geneve.max_header_size)
         # Make sure that domain_name is not included.
         expected_dhcp_options_rows = {
             'cidr': '10.0.0.0/24',
             'external_ids': {'subnet_id': subnet['id']},
             'options': {'dns_server': '{10.10.10.10}',
                         'lease_time': '43200',
-                        'mtu': '1450',
+                        'mtu': mtu,
                         'router': '10.0.0.1',
                         'server_id': '10.0.0.1',
                         'server_mac': dhcp_mac}}
@@ -824,7 +841,7 @@ class TestPortSecurity(base.TestOVNFunctionalBase):
 
     def _verify_port_acls(self, port_id, expected_acls):
         port_acls = self._get_port_related_acls(port_id)
-        self.assertItemsEqual(expected_acls, port_acls)
+        self.assertCountEqual(expected_acls, port_acls)
 
     def test_port_security_port_group(self):
         n1 = self._make_network(self.fmt, 'n1', True)
@@ -906,6 +923,31 @@ class TestPortSecurity(base.TestOVNFunctionalBase):
         self._verify_port_acls(port_id, expected_acls_with_sg_ps_enabled)
 
 
+class TestSecurityGroups(base.TestOVNFunctionalBase):
+
+    def test_security_group_creation_and_deletion(self):
+        sg = self._make_security_group(self.fmt)['security_group']
+        rev_num = rev_db.get_revision_row(self.context, sg['id'])
+        self.assertEqual(1, rev_num.revision_number)
+        for sg_rule in sg['security_group_rules']:
+            rev_num = rev_db.get_revision_row(self.context, sg_rule['id'])
+            self.assertEqual(0, rev_num.revision_number)
+
+        # Retrieve the ACL UUIDs before deleting the Port_Group; this operation
+        # will also delete the associated ACLs.
+        pg_name = utils.ovn_port_group_name(sg['id'])
+        pg = self.nb_api.pg_get(pg_name).execute(check_errors=True)
+        acl_uuids = [acl.uuid for acl in pg.acls]
+        self._delete('security-groups', sg['id'])
+        self.assertIsNone(rev_db.get_revision_row(self.context, sg['id']))
+        for sg_rule in sg['security_group_rules']:
+            self.assertIsNone(rev_db.get_revision_row(self.context,
+                                                      sg_rule['id']))
+        for acl_uuid in acl_uuids:
+            self.assertIsNone(
+                self.nb_api.lookup('ACL', acl_uuid, default=None))
+
+
 class TestDNSRecords(base.TestOVNFunctionalBase):
     _extension_drivers = ['port_security', 'dns']
 
@@ -915,7 +957,7 @@ class TestDNSRecords(base.TestOVNFunctionalBase):
             observed_dns_records.append(
                 {'external_ids': dns_row.external_ids,
                  'records': dns_row.records})
-        self.assertItemsEqual(expected_dns_records, observed_dns_records)
+        self.assertCountEqual(expected_dns_records, observed_dns_records)
 
     def _validate_ls_dns_records(self, lswitch_name, expected_dns_records):
         ls = idlutils.row_by_value(self.nb_api.idl,
@@ -925,11 +967,11 @@ class TestDNSRecords(base.TestOVNFunctionalBase):
             observed_dns_records.append(
                 {'external_ids': dns_row.external_ids,
                  'records': dns_row.records})
-        self.assertItemsEqual(expected_dns_records, observed_dns_records)
+        self.assertCountEqual(expected_dns_records, observed_dns_records)
 
     def setUp(self):
         ovn_config.cfg.CONF.set_override('dns_domain', 'ovn.test')
-        super(TestDNSRecords, self).setUp()
+        super().setUp()
 
     def test_dns_records(self):
         expected_dns_records = []
@@ -964,6 +1006,9 @@ class TestDNSRecords(base.TestOVNFunctionalBase):
              'records': {'n1p1': port_ips, 'n1p1.ovn.test': port_ips,
                          'n1p1.net-n1': port_ips}}
         ]
+        for ip in port_ips.split(" "):
+            p_record = netaddr.IPAddress(ip).reverse_dns.rstrip(".")
+            expected_dns_records[0]['records'][p_record] = 'n1p1.ovn.test'
 
         self._validate_dns_records(expected_dns_records)
         self._validate_ls_dns_records(n1_lswitch_name,
@@ -989,6 +1034,9 @@ class TestDNSRecords(base.TestOVNFunctionalBase):
         expected_dns_records.append(
             {'external_ids': {'ls_name': n2_lswitch_name},
              'records': {'n2p1': port_ips, 'n2p1.ovn.test': port_ips}})
+        for ip in port_ips.split(" "):
+            p_record = netaddr.IPAddress(ip).reverse_dns.rstrip(".")
+            expected_dns_records[1]['records'][p_record] = 'n2p1.ovn.test'
         self._validate_dns_records(expected_dns_records)
         self._validate_ls_dns_records(n1_lswitch_name,
                                       [expected_dns_records[0]])
@@ -1006,6 +1054,9 @@ class TestDNSRecords(base.TestOVNFunctionalBase):
         expected_dns_records[0]['records']['n1p2'] = port_ips
         expected_dns_records[0]['records']['n1p2.ovn.test'] = port_ips
         expected_dns_records[0]['records']['n1p2.net-n1'] = port_ips
+        for ip in port_ips.split(" "):
+            p_record = netaddr.IPAddress(ip).reverse_dns.rstrip(".")
+            expected_dns_records[0]['records'][p_record] = 'n1p2.ovn.test'
         self._validate_dns_records(expected_dns_records)
         self._validate_ls_dns_records(n1_lswitch_name,
                                       [expected_dns_records[0]])
@@ -1019,6 +1070,11 @@ class TestDNSRecords(base.TestOVNFunctionalBase):
         res = req.get_response(self.api)
         self.assertEqual(200, res.status_int)
         expected_dns_records[0]['records'].pop('n1p1')
+        port_ips = " ".join([f['ip_address']
+                             for f in n1p1['port']['fixed_ips']])
+        for ip in port_ips.split(" "):
+            p_record = netaddr.IPAddress(ip).reverse_dns.rstrip(".")
+            expected_dns_records[0]['records'].pop(p_record)
         expected_dns_records[0]['records'].pop('n1p1.ovn.test')
         expected_dns_records[0]['records'].pop('n1p1.net-n1')
         self._validate_dns_records(expected_dns_records)
