@@ -18,23 +18,31 @@ import functools
 import re
 import time
 from unittest import mock
+import uuid
 
 import netaddr
 from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import portbindings
+from neutron_lib.api.definitions import qinq as qinq_apidef
+from neutron_lib.api.definitions import vlantransparent as vlan_apidef
 from neutron_lib import constants
+from neutron_lib.db import api as db_api
 from neutron_lib.exceptions import agent as agent_exc
 from oslo_config import cfg
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import event
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
 from neutron.common import utils as n_utils
+from neutron.conf import common as common_conf
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
+from neutron.db import ovn_hash_ring_db
 from neutron.db import ovn_revision_numbers_db as db_rev
+from neutron.plugins.ml2 import db as ml2_db
+from neutron.plugins.ml2.drivers.ovn.agent import neutron_agent as n_agent
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
-from neutron.tests import base as tests_base
 from neutron.tests.functional import base
 from neutron.tests.unit.extensions import test_securitygroup as test_sg
 
@@ -57,6 +65,69 @@ VHOSTUSER_VIF_DETAILS = {
 }
 
 
+class TestOVNMechanismDriver(base.TestOVNFunctionalBase):
+
+    def test__init_hash_ring(self):
+        # Create a differentiated OVN hash ring name.
+        ring_group = uuidutils.generate_uuid()
+        self.mech_driver.hash_ring_group = ring_group
+
+        # Create several OVN hash registers left by a previous execution.
+        created_at = timeutils.utcnow() - datetime.timedelta(1)
+        node_uuids = [uuidutils.generate_uuid(),
+                      uuidutils.generate_uuid(),
+                      uuidutils.generate_uuid()]
+        with db_api.CONTEXT_WRITER.using(self.context):
+            for idx in range(3):
+                ovn_hash_ring_db.add_node(
+                    self.context, ring_group,
+                    node_uuids[idx], created_at=created_at)
+
+        # Check the existing OVN hash ring registers.
+        ovn_hrs = ovn_hash_ring_db.get_nodes(self.context, ring_group)
+        self.assertEqual(3, len(ovn_hrs))
+
+        start_time = timeutils.utcnow()
+        self.mech_driver._start_time = int(start_time.timestamp())
+        for idx in range(3):
+            self.mech_driver._node_uuid = node_uuids[idx]
+            self.mech_driver._init_hash_ring(self.context)
+
+        ovn_hrs = ovn_hash_ring_db.get_nodes(self.context, ring_group)
+        self.assertEqual(3, len(ovn_hrs))
+        for ovn_hr in ovn_hrs:
+            self.assertEqual(int(start_time.timestamp()),
+                             ovn_hr.created_at.timestamp())
+
+
+class TestOvsdbPersistUuid(base.TestOVNFunctionalBase):
+
+    def _create_port(self, name, net_id):
+        data = {'port': {'name': name,
+                         'tenant_id': self._tenant_id,
+                         'network_id': net_id}}
+
+        req = self.new_create_request('ports', data, self.fmt)
+        res = req.get_response(self.api)
+        return self.deserialize(self.fmt, res)['port']['id']
+
+    def test_old_network_new_port(self):
+        if not utils.ovs_persist_uuid_supported(self.nb_api):
+            self.skipTest("OVS persist_uuid not supported")
+        mock_supported = mock.patch.object(utils, 'ovs_persist_uuid_supported',
+                                           return_value=False).start()
+        network = self._make_network(self.fmt, 'n1', True)
+        network_id = network['network']['id']
+        self._create_subnet(self.fmt, network_id, '10.0.0.0/24')
+        n1_ls = self.nb_api.ls_get(
+            utils.ovn_name(network_id)).execute(check_error=True)
+        self.assertNotEqual(uuid.UUID(network_id), n1_ls.uuid)
+        mock_supported.return_value = True
+        port = self._create_port("n1_port", network_id)
+        port_lsp = self.nb_api.lsp_get(port).execute(check_error=True)
+        self.assertIn(port_lsp, n1_ls.ports)
+
+
 class TestPortBinding(base.TestOVNFunctionalBase):
 
     def setUp(self, **kwargs):
@@ -260,6 +331,61 @@ class TestPortBinding(base.TestOVNFunctionalBase):
             ovn_const.LSP_OPTIONS_VIF_PLUG_REPRESENTOR_VF_NUM_KEY: str(vf_num),
         })
 
+    def _create_router(self):
+        e1 = self._make_network(self.fmt, 'e1', True, as_admin=True,
+                                arg_list=('router:external',
+                                          'provider:network_type',
+                                          'provider:physical_network'),
+                                **{'router:external': True,
+                                   'provider:network_type': 'flat',
+                                   'provider:physical_network': 'public'})
+        res = self._create_subnet(self.fmt, e1['network']['id'],
+                                  '100.0.0.0/24')
+        s1 = self.deserialize(self.fmt, res)
+        return self.l3_plugin.create_router(
+            self.context,
+            {'router': {
+                'name': uuidutils.generate_uuid(),
+                'admin_state_up': True,
+                'tenant_id': self._tenant_id,
+                'external_gateway_info': {
+                    'enable_snat': True,
+                    'network_id': e1['network']['id'],
+                    'external_fixed_ips': [
+                        {'ip_address': '100.0.0.2',
+                         'subnet_id': s1['subnet']['id']}]}}})
+
+    def test_lsp_up_event_update_and_creation(self):
+        def _check_set_up_called(_mock, lsp_name):
+            n_utils.wait_until_true(lambda: _mock.call_count == 1, timeout=10)
+            mock_set_up.assert_called_once_with(lsp_name)
+
+        with mock.patch.object(self.mech_driver, 'set_port_status_up') as \
+                mock_set_up:
+            router = self._create_router()
+            lr = self.nb_api.lr_get(utils.ovn_name(router['id'])).execute(
+                check_error=True)
+            lrp = self.nb_api.lrp_get(lr.ports[0].uuid).execute(
+                check_error=True)
+            lsp_name = lrp.name.replace('lrp-', '')
+            # Check that ``set_port_status_up`` has been called when the port
+            # is created.
+            _check_set_up_called(mock_set_up, lsp_name)
+
+            # Restart the OVS database that forces the IDL reconnection.
+            # Reset the ``mock_set_up`` that needs to be called again.
+            with mock.patch.object(self.mech_driver, '_init_hash_ring'), \
+                    mock.patch.object(ml2_db, 'get_port') as mock_get_port:
+                mock_get_port.return_value = mock.Mock(
+                    port_bindings=[mock.Mock(host='host1')],
+                    id=lsp_name, device_owner='router')
+                mock_set_up.reset_mock()
+                self.restart(delete_dbs=False)
+
+            # Check that ``set_port_status_up`` has been called when the IDL
+            # reconnects.
+            _check_set_up_called(mock_set_up, lsp_name)
+
 
 class TestPortBindingOverTcp(TestPortBinding):
     def get_ovsdb_server_protocol(self):
@@ -275,7 +401,7 @@ class TestPortBindingOverSsl(TestPortBinding):
 class TestNetworkMTUUpdate(base.TestOVNFunctionalBase):
 
     def setUp(self):
-        super(TestNetworkMTUUpdate, self).setUp()
+        super().setUp()
         self._ovn_client = self.mech_driver._ovn_client
         self.n1 = self._make_network(self.fmt, 'n1', True)
         res = self._create_subnet(self.fmt, self.n1['network']['id'],
@@ -323,7 +449,7 @@ class TestNetworkMTUUpdate(base.TestOVNFunctionalBase):
 class TestVirtualPorts(base.TestOVNFunctionalBase):
 
     def setUp(self):
-        super(TestVirtualPorts, self).setUp()
+        super().setUp()
         self._ovn_client = self.mech_driver._ovn_client
         self.n1 = self._make_network(self.fmt, 'n1', True)
         res = self._create_subnet(self.fmt, self.n1['network']['id'],
@@ -375,7 +501,6 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
         check = functools.partial(self._is_ovn_port_type, port_id, type)
         n_utils.wait_until_true(check, timeout=10)
 
-    @tests_base.unstable_test("bug 1865453")
     def test_virtual_port_created_before(self):
         virt_port = self._create_port()
         virt_ip = virt_port['fixed_ips'][0]['ip_address']
@@ -411,7 +536,6 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
             backup['id'],
             ovn_vport.options[ovn_const.LSP_OPTIONS_VIRTUAL_PARENTS_KEY])
 
-    @tests_base.unstable_test("bug 1865453")
     def test_virtual_port_update_address_pairs(self):
         primary = self._create_port()
         backup = self._create_port()
@@ -420,7 +544,7 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
 
         # Assert the virt port does not yet have the type virtual (no
         # address pairs were set yet)
-        self._check_port_type(virt_port['id'], ''),
+        self._check_port_type(virt_port['id'], '')
         ovn_vport = self._find_port_row(virt_port['id'])
         self.assertNotIn(ovn_const.LSP_OPTIONS_VIRTUAL_PARENTS_KEY,
                          ovn_vport.options)
@@ -431,7 +555,7 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
         self._set_allowed_address_pair(primary['id'], virt_ip)
 
         # Assert the virt port is now updated
-        self._check_port_type(virt_port['id'], ovn_const.LSP_TYPE_VIRTUAL),
+        self._check_port_type(virt_port['id'], ovn_const.LSP_TYPE_VIRTUAL)
         ovn_vport = self._find_port_row(virt_port['id'])
         self.assertEqual(
             virt_ip,
@@ -444,7 +568,7 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
         self._set_allowed_address_pair(backup['id'], virt_ip)
 
         # Assert the virt port now includes the backup port as a parent
-        self._check_port_type(virt_port['id'], ovn_const.LSP_TYPE_VIRTUAL),
+        self._check_port_type(virt_port['id'], ovn_const.LSP_TYPE_VIRTUAL)
         ovn_vport = self._find_port_row(virt_port['id'])
         self.assertEqual(
             virt_ip,
@@ -460,7 +584,7 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
         self._unset_allowed_address_pair(primary['id'])
 
         # Assert the virt port now only has the backup port as a parent
-        self._check_port_type(virt_port['id'], ovn_const.LSP_TYPE_VIRTUAL),
+        self._check_port_type(virt_port['id'], ovn_const.LSP_TYPE_VIRTUAL)
         ovn_vport = self._find_port_row(virt_port['id'])
         self.assertEqual(
             virt_ip,
@@ -474,14 +598,13 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
 
         # Assert the virt port is not type virtual anymore and the virtual
         # port options are cleared
-        self._check_port_type(virt_port['id'], ''),
+        self._check_port_type(virt_port['id'], '')
         ovn_vport = self._find_port_row(virt_port['id'])
         self.assertNotIn(ovn_const.LSP_OPTIONS_VIRTUAL_PARENTS_KEY,
                          ovn_vport.options)
         self.assertNotIn(ovn_const.LSP_OPTIONS_VIRTUAL_IP_KEY,
                          ovn_vport.options)
 
-    @tests_base.unstable_test("bug 1865453")
     def test_virtual_port_created_after(self):
         primary = self._create_port(fixed_ip='10.0.0.11')
         backup = self._create_port(fixed_ip='10.0.0.12')
@@ -508,7 +631,6 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
             backup['id'],
             ovn_vport.options[ovn_const.LSP_OPTIONS_VIRTUAL_PARENTS_KEY])
 
-    @tests_base.unstable_test("bug 1865453")
     def test_virtual_port_delete_parents(self):
         primary = self._create_port()
         backup = self._create_port()
@@ -585,7 +707,7 @@ class TestVirtualPorts(base.TestOVNFunctionalBase):
 class TestExternalPorts(base.TestOVNFunctionalBase):
 
     def setUp(self):
-        super(TestExternalPorts, self).setUp()
+        super().setUp()
         self._ovn_client = self.mech_driver._ovn_client
         self.n1 = self._make_network(self.fmt, 'n1', True)
         res = self._create_subnet(self.fmt, self.n1['network']['id'],
@@ -642,7 +764,7 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
                 self.count = 0
                 table = 'Logical_Switch_Port'
                 events = (self.ROW_UPDATE,)
-                super(LogicalSwitchPortUpdateUpEventTest, self).__init__(
+                super().__init__(
                     events, table, (('up', '=', True),),
                     old_conditions=(('up', '=', False),))
 
@@ -657,7 +779,7 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
                 self.count = 0
                 table = 'Logical_Switch_Port'
                 events = (self.ROW_UPDATE,)
-                super(LogicalSwitchPortUpdateDownEventTest, self).__init__(
+                super().__init__(
                     events, table, (('up', '=', False),),
                     old_conditions=(('up', '=', True),))
 
@@ -848,7 +970,7 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
 class TestSecurityGroup(base.TestOVNFunctionalBase):
 
     def setUp(self):
-        super(TestSecurityGroup, self).setUp()
+        super().setUp()
         self._ovn_client = self.mech_driver._ovn_client
         self.plugin = self.mech_driver._plugin
         self.sg_data = {
@@ -888,7 +1010,7 @@ class TestSecurityGroup(base.TestOVNFunctionalBase):
         def check_acl_actions(sg_id, expected):
             self.assertEqual(
                 {expected},
-                set(a.action for a in self._find_acls_for_sg(sg_id))
+                {a.action for a in self._find_acls_for_sg(sg_id)}
             )
 
         sg = self.plugin.create_security_group(
@@ -943,7 +1065,7 @@ class TestSecurityGroup(base.TestOVNFunctionalBase):
 class TestProvnetPorts(base.TestOVNFunctionalBase):
 
     def setUp(self):
-        super(TestProvnetPorts, self).setUp()
+        super().setUp()
         self._ovn_client = self.mech_driver._ovn_client
 
     def _find_port_row_by_name(self, name):
@@ -1015,6 +1137,87 @@ class TestProvnetPorts(base.TestOVNFunctionalBase):
         self.assertIsNone(ovn_localnetport)
 
 
+class TestVlanTransparencyOptions(base.TestOVNFunctionalBase):
+
+    """Tests for the vlan_transparent and vlan_qinq network params.
+
+    This class contains tests which tests both "vlan_transparent" and
+    "vlan_qinq" options. The reason why those 2 are tested together is that
+    both options are using the same options on the OVN side and are mutually
+    exclusive.
+    """
+
+    def setUp(self):
+        common_conf.register_core_common_config_opts()
+        super().setUp()
+        self._ovn_client = self.mech_driver._ovn_client
+
+    def _find_row_by_name(self, row_name, name):
+        cmd = self.nb_api.db_find_rows(row_name, ('name', '=', name))
+        rows = cmd.execute(check_error=True)
+        return rows[0] if rows else None
+
+    def _find_port_row_by_name(self, name):
+        return self._find_row_by_name('Logical_Switch_Port', name)
+
+    def _find_network_row_by_name(self, name):
+        return self._find_row_by_name('Logical_Switch', name)
+
+    def _test_network_with_qinq_and_vlan_transparent(
+            self, vlan_qinq, vlan_transparent):
+        net = self._make_network(
+            self.fmt, 'n1', True, as_admin=True,
+            arg_list=('provider:network_type',
+                      'provider:segmentation_id',
+                      'provider:physical_network',
+                      qinq_apidef.QINQ_FIELD,
+                      vlan_apidef.VLANTRANSPARENT),
+            **{'provider:network_type': 'vlan',
+               'provider:segmentation_id': 100,
+               'provider:physical_network': 'physnet1',
+               qinq_apidef.QINQ_FIELD: vlan_qinq,
+               vlan_apidef.VLANTRANSPARENT: vlan_transparent}
+        )['network']
+
+        seg_db = self.segments_plugin.get_segments(
+            self.context, filters={'network_id': [net['id']]})
+        ovn_network = self._find_network_row_by_name(utils.ovn_name(net['id']))
+        ovn_localnetport = self._find_port_row_by_name(
+            utils.ovn_provnet_port_name(seg_db[0]['id']))
+
+        if vlan_qinq:
+            self.assertEqual(ovn_const.ETHTYPE_8021ad,
+                             ovn_localnetport.options[ovn_const.VLAN_ETHTYPE])
+            # This means that "vlan-passthru" should be set to "true" for
+            # the LS
+            self.assertEqual('true',
+                             ovn_network.other_config[ovn_const.VLAN_PASSTHRU])
+        else:
+            self.assertNotIn(ovn_const.VLAN_ETHTYPE, ovn_localnetport.options)
+            if vlan_transparent:
+                # This means that "vlan-passthru" should be set to "true" for
+                # the LS, in case of vlan-transparent network there is no need
+                # to set ethtype on the localnet port as 802.1q is default
+                # value for this field in the OVN db
+                self.assertEqual(
+                    'true', ovn_network.other_config[ovn_const.VLAN_PASSTHRU])
+            else:
+                self.assertEqual(
+                    'false', ovn_network.other_config[ovn_const.VLAN_PASSTHRU])
+
+    def test_network_with_qinq_enabled_vlan_transparent_disabled(self):
+        self._test_network_with_qinq_and_vlan_transparent(
+            True, False)
+
+    def test_network_with_qinq_disabled_vlan_transparent_enabled(self):
+        self._test_network_with_qinq_and_vlan_transparent(
+            False, True)
+
+    def test_network_with_qinq_and_vlan_transparent_disabled(self):
+        self._test_network_with_qinq_and_vlan_transparent(
+            False, False)
+
+
 class TestMetadataPorts(base.TestOVNFunctionalBase):
 
     def setUp(self, *args, **kwargs):
@@ -1174,7 +1377,8 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         agent_event = AgentWaitEvent(self.mech_driver, [chassis_name])
         self.sb_api.idl.notify_handler.watch_event(agent_event)
 
-        self.chassis = self.add_fake_chassis(self.host, name=chassis_name,
+        self.chassis = self.add_fake_chassis(
+            self.host, name=chassis_name,
             external_ids={
                 ovn_const.OVN_AGENT_METADATA_ID_KEY: metadata_agent_id})
 
@@ -1200,11 +1404,10 @@ class TestAgentApi(base.TestOVNFunctionalBase):
     def _check_chassis_registers(self, present=True):
         chassis = self.sb_api.lookup('Chassis', self.chassis, default=None)
         chassis_name = chassis.name if chassis else None
-        if self.sb_api.is_table_present('Chassis_Private'):
-            ch_private = self.sb_api.lookup(
-                'Chassis_Private', self.chassis, default=None)
-            ch_private_name = ch_private.name if ch_private else None
-            self.assertEqual(chassis_name, ch_private_name)
+        ch_private = self.sb_api.lookup(
+            'Chassis_Private', self.chassis, default=None)
+        ch_private_name = ch_private.name if ch_private else None
+        self.assertEqual(chassis_name, ch_private_name)
         if present:
             self.assertEqual(self.chassis, chassis_name)
         else:
@@ -1248,6 +1451,13 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         self.assertCountEqual(list(self.agent_types.values()), agent_ids)
 
     def test_agent_delete(self):
+        agent_cache = n_agent.AgentCache(mock.ANY)
+
+        def wait_until_removed(agent_id):
+            n_utils.wait_until_true(
+                lambda: agent_cache.agents.get(agent_id) is None,
+                timeout=5)
+
         # Non OVN agent deletion.
         agent_id = self.agent_types[self.TEST_AGENT]
         self.plugin.delete_agent(self.context, agent_id)
@@ -1263,6 +1473,7 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         self._check_chassis_registers()
         self.plugin.delete_agent(self.context, controller_id)
         self._check_chassis_registers(present=False)
+        wait_until_removed(controller_id)
         self.assertRaises(agent_exc.AgentNotFound, self.plugin.get_agent,
                           self.context, controller_id)
         self.assertEqual(
@@ -1270,6 +1481,7 @@ class TestAgentApi(base.TestOVNFunctionalBase):
             self.plugin.get_agent(self.context, metadata_id)['id'])
 
         self.plugin.delete_agent(self.context, metadata_id)
+        wait_until_removed(metadata_id)
         self.assertRaises(agent_exc.AgentNotFound, self.plugin.get_agent,
                           self.context, metadata_id)
 
@@ -1339,18 +1551,18 @@ class TestNATRuleGatewayPort(_TestRouter):
                'provider:network_type': 'flat',
                'provider:physical_network': 'public'})['network']
         res = self._create_subnet(self.fmt, ext_net['id'],
-            '100.0.0.0/24', gateway_ip='100.0.0.254',
-            allocation_pools=[{'start': '100.0.0.2',
-                               'end': '100.0.0.253'}],
-            enable_dhcp=False)
+                                  '100.0.0.0/24', gateway_ip='100.0.0.254',
+                                  allocation_pools=[{'start': '100.0.0.2',
+                                                     'end': '100.0.0.253'}],
+                                  enable_dhcp=False)
         ext_subnet = self.deserialize(self.fmt, res)['subnet']
         net1 = self._make_network(
             self.fmt, 'network1test', True)['network']
         res = self._create_subnet(self.fmt, net1['id'],
-            '192.168.0.0/24', gateway_ip='192.168.0.1',
-            allocation_pools=[{'start': '192.168.0.2',
-                               'end': '192.168.0.253'}],
-            enable_dhcp=False)
+                                  '192.168.0.0/24', gateway_ip='192.168.0.1',
+                                  allocation_pools=[{'start': '192.168.0.2',
+                                                     'end': '192.168.0.253'}],
+                                  enable_dhcp=False)
         subnet1 = self.deserialize(self.fmt, res)['subnet']
         external_gateway_info = {
             'enable_snat': True,
@@ -1411,10 +1623,10 @@ class TestRouterGWPort(_TestRouter):
             self.fmt, 'inner_network', True)['network']
         subnet_cidr = '192.168.0.0/24'
         res = self._create_subnet(self.fmt, inner_network['id'],
-            '192.168.0.0/24', gateway_ip='192.168.0.1',
-            allocation_pools=[{'start': '192.168.0.2',
-                               'end': '192.168.0.253'}],
-            enable_dhcp=False)
+                                  '192.168.0.0/24', gateway_ip='192.168.0.1',
+                                  allocation_pools=[{'start': '192.168.0.2',
+                                                     'end': '192.168.0.253'}],
+                                  enable_dhcp=False)
         inner_subnet = self.deserialize(self.fmt, res)['subnet']
         self._add_router_interface(router['id'], inner_subnet['id'])
 
@@ -1433,7 +1645,7 @@ class TestRouterGWPort(_TestRouter):
         nats = lr.nat
         self.assertEqual(1, len(nats))
         expected_logical_ip = (
-            ovn_const.OVN_DEFAULT_SNAT_CIDR if nested_snat else subnet_cidr
+            constants.IPv4_ANY if nested_snat else subnet_cidr
         )
         self.assertEqual(expected_logical_ip, nats[0].logical_ip)
 
