@@ -19,7 +19,6 @@ import unittest
 
 import fixtures
 import netaddr
-from neutron_lib.callbacks import priority_group
 from neutron_lib import constants
 from neutron_lib.services.logapi import constants as log_const
 from neutron_lib.utils import helpers
@@ -28,16 +27,6 @@ from oslo_utils import netutils
 from oslo_utils import timeutils
 
 
-# NOTE(ykarel): from neutron-lib 3.9.0, cancellable flag was added
-# test the existence of the is_cancellable_event function to check if
-# cancellable flag is supported or not. This compatibility check can
-# be removed once neutron-lib >= 3.9.0 in requirements.txt.
-_CANCELLABLE_FLAG_SUPPORTED = True
-try:
-    from neutron_lib.callbacks.events import is_cancellable_event  # noqa
-except ImportError:
-    _CANCELLABLE_FLAG_SUPPORTED = False
-
 LAST_RANDOM_PORT_RANGE_GENERATED = 1
 
 
@@ -146,18 +135,6 @@ def make_mock_plugin_json_encodable(plugin_instance_mock):
         method_mock._get_child_mock = _get_child_mock
 
 
-def get_subscribe_args(*args):
-    args = list(args)  # don't modify original list
-    args.append(priority_group.PRIORITY_DEFAULT)
-    # NOTE(ykarel): from neutron-lib 3.9.0, cancellable flag was added.
-    # old signature: (callback, resource, event, priority=PRIORITY_DEFAULT)
-    # new signature: (callback, resource, event, priority=PRIORITY_DEFAULT,
-    # cancellable=False)
-    if len(args) == 4 and _CANCELLABLE_FLAG_SUPPORTED:
-        args.append(False)
-    return args
-
-
 def fail(msg=None):
     """Fail immediately, with the given message.
 
@@ -221,12 +198,9 @@ def get_random_ip_address(version=4):
                                      random.randint(3, 254),
                                      random.randint(3, 254))
         return netaddr.IPAddress(ip_string)
-    else:
-        ip = netutils.get_ipv6_addr_by_EUI64(
-            '2001:db8::/64',
-            net.get_random_mac(['fe', '16', '3e', '00', '00', '00'])
-        )
-        return ip
+    return netutils.get_ipv6_addr_by_EUI64(
+        '2001:db8::/64',
+        net.get_random_mac(['fe', '16', '3e', '00', '00', '00']))
 
 
 def get_random_router_status():
@@ -282,3 +256,7 @@ def get_random_security_event():
 
 def get_random_port_numa_affinity_policy():
     return random.choice(constants.PORT_NUMA_POLICIES)
+
+
+def get_random_port_hardware_offload_type():
+    return random.choice(constants.VALID_HWOL_TYPES)
