@@ -24,6 +24,7 @@ from oslo_utils import uuidutils
 import psutil
 
 from neutron.agent.linux import external_process as ep
+from neutron.agent.linux import ip_lib
 from neutron.common import utils as common_utils
 from neutron.tests import base
 
@@ -46,7 +47,7 @@ DEFAULT_ENVVAR = ep.PROCESS_TAG + '=' + ep.DEFAULT_SERVICE_NAME
 class BaseTestProcessMonitor(base.BaseTestCase):
 
     def setUp(self):
-        super(BaseTestProcessMonitor, self).setUp()
+        super().setUp()
         self.log_patch = mock.patch("neutron.agent.linux.external_process."
                                     "LOG.error")
         self.error_log = self.log_patch.start()
@@ -117,7 +118,7 @@ class TestProcessMonitor(BaseTestProcessMonitor):
 
 class TestProcessManager(base.BaseTestCase):
     def setUp(self):
-        super(TestProcessManager, self).setUp()
+        super().setUp()
         self.execute_p = mock.patch('neutron.agent.common.utils.execute')
         self.execute = self.execute_p.start()
         self.delete_if_exists = mock.patch(
@@ -269,10 +270,11 @@ class TestProcessManager(base.BaseTestCase):
                 active.__get__ = mock.Mock(return_value=True)
                 manager = ep.ProcessManager(self.conf, 'uuid')
 
-                with mock.patch.object(ep, 'utils') as utils:
+                with mock.patch.object(ip_lib.IpNetnsCommand, 'execute') as \
+                        mock_execute:
                     manager.disable()
                     env = {ep.PROCESS_TAG: ep.DEFAULT_SERVICE_NAME + '-uuid'}
-                    utils.assert_has_calls([
+                    mock_execute.assert_has_calls([
                         mock.call.execute(['kill', '-9', 4],
                                           addl_env=env,
                                           run_as_root=False,
@@ -286,10 +288,11 @@ class TestProcessManager(base.BaseTestCase):
 
                 manager = ep.ProcessManager(self.conf, 'uuid', namespace='ns')
 
-                with mock.patch.object(ep, 'utils') as utils:
+                with mock.patch.object(ip_lib.IpNetnsCommand, 'execute') as \
+                        mock_execute:
                     manager.disable()
                     env = {ep.PROCESS_TAG: ep.DEFAULT_SERVICE_NAME + '-uuid'}
-                    utils.assert_has_calls([
+                    mock_execute.assert_has_calls([
                         mock.call.execute(
                             ['kill', '-9', 4], addl_env=env, run_as_root=True,
                             privsep_exec=True)])
@@ -331,12 +334,12 @@ class TestProcessManager(base.BaseTestCase):
                 manager = ep.ProcessManager(
                     self.conf, 'uuid', namespace=namespace,
                     service=service_name)
-                with mock.patch.object(ep, 'utils') as utils, \
-                        mock.patch.object(os.path, 'isfile',
-                                          return_value=kill_script_exists):
+                with mock.patch.object(ip_lib.IpNetnsCommand, 'execute') as \
+                        execute_mock, mock.patch.object(
+                        os.path, 'isfile', return_value=kill_script_exists):
                     manager.disable()
                     addl_env = {ep.PROCESS_TAG: service_name + '-uuid'}
-                    utils.execute.assert_called_with(
+                    execute_mock.assert_called_with(
                         expected_cmd, addl_env=addl_env,
                         run_as_root=bool(namespace), privsep_exec=True)
 
@@ -454,7 +457,7 @@ class TestProcessManagerScript(TestProcessManager):
             active.__get__ = mock.Mock(return_value=False)
             pm.enable()
 
-        with open(output.name, 'r') as f:
+        with open(output.name) as f:
             ret_value = f.readline().strip()
         expected_value = ('Variable PROCESS_TAG set: %s-%s' %
                           (service_name, uuid))
@@ -475,7 +478,7 @@ class TestProcessManagerScript(TestProcessManager):
             mock_kill_cmd.return_value = [script.name, output.name]
             pm.disable(sig='15')
 
-        with open(output.name, 'r') as f:
+        with open(output.name) as f:
             ret_value = f.readline().strip()
         expected_value = ('Variable PROCESS_TAG set: %s-%s' %
                           (service_name, uuid))
