@@ -23,6 +23,7 @@ from oslo_utils import uuidutils
 
 from neutron.agent.linux import external_process as ep
 from neutron.agent.linux import utils as linux_utils
+from neutron.agent.metadata import driver_base
 from neutron.agent.ovn.metadata import agent as metadata_agent
 from neutron.agent.ovn.metadata import driver as metadata_driver
 from neutron.common import metadata as comm_meta
@@ -40,6 +41,7 @@ class TestMetadataDriverProcess(base.BaseTestCase):
     EUNAME = 'neutron'
     EGNAME = 'neutron'
     METADATA_DEFAULT_IP = '169.254.169.254'
+    METADATA_DEFAULT_IPV6 = 'fe80::a9fe:a9fe'
     METADATA_PORT = 8080
     METADATA_SOCKET = '/socket/path'
     PIDFILE = 'pidfile'
@@ -51,7 +53,7 @@ class TestMetadataDriverProcess(base.BaseTestCase):
     }
 
     def setUp(self):
-        super(TestMetadataDriverProcess, self).setUp()
+        super().setUp()
         mock.patch('eventlet.spawn').start()
         self.delete_if_exists = mock.patch.object(linux_utils,
                                                   'delete_if_exists').start()
@@ -99,27 +101,39 @@ class TestMetadataDriverProcess(base.BaseTestCase):
                     'neutron.agent.linux.external_process.'
                     'ProcessManager.active',
                     new_callable=mock.PropertyMock,
-                    side_effect=[False, True]):
+                    side_effect=[False, True]),\
+                mock.patch(
+                    'neutron.agent.linux.ip_lib.'
+                    'IpAddrCommand.wait_until_address_ready',
+                    return_value=True),\
+                mock.patch.object(driver_base.HaproxyConfiguratorBase,
+                                  'is_config_file_obsolete',
+                                  return_value=False):
             cfg_file = os.path.join(
                 metadata_driver.HaproxyConfigurator.get_config_path(
-                    cfg.CONF.state_path),
+                    agent.conf.state_path),
                 "%s.conf" % datapath_id)
             mock_open = self.useFixture(
                 lib_fixtures.OpenFixture(cfg_file)).mock_open
-            metadata_driver.MetadataDriver.spawn_monitored_metadata_proxy(
+            bind_v6_line = 'bind {}:{} interface {}'.format(
+                self.METADATA_DEFAULT_IPV6, self.METADATA_PORT, 'fake-if')
+            proxy = metadata_driver.MetadataDriver()
+            proxy.spawn_monitored_metadata_proxy(
                 agent._process_monitor,
                 metadata_ns,
                 self.METADATA_PORT,
-                cfg.CONF,
+                agent.conf,
                 bind_address=self.METADATA_DEFAULT_IP,
-                network_id=datapath_id)
+                network_id=datapath_id,
+                bind_address_v6=self.METADATA_DEFAULT_IPV6,
+                bind_interface='fake-if')
 
             netns_execute_args = [
                 service_name,
                 '-f', cfg_file]
 
             log_tag = '{}-{}-{}'.format(
-                service_name, metadata_driver.METADATA_SERVICE_NAME,
+                service_name, driver_base.METADATA_SERVICE_NAME,
                 datapath_id)
 
             expected_params = {
@@ -133,7 +147,7 @@ class TestMetadataDriverProcess(base.BaseTestCase):
                 'pidfile': self.PIDFILE,
                 'log_level': 'debug',
                 'log_tag': log_tag,
-                'bind_v6_line': ''}
+                'bind_v6_line': bind_v6_line}
 
             if rate_limited:
                 expected_params.update(self.RATE_LIMIT_CONFIG,
@@ -146,7 +160,7 @@ class TestMetadataDriverProcess(base.BaseTestCase):
             else:
                 expected_config_template = (
                     comm_meta.METADATA_HAPROXY_GLOBAL +
-                    metadata_driver._UNLIMITED_CONFIG_TEMPLATE +
+                    driver_base._UNLIMITED_CONFIG_TEMPLATE +
                     metadata_driver._HEADER_CONFIG_TEMPLATE)
 
             cfg_contents = expected_config_template % expected_params
@@ -165,7 +179,7 @@ class TestMetadataDriverProcess(base.BaseTestCase):
             self.delete_if_exists.assert_called_once_with(
                 mock.ANY, run_as_root=True)
 
-    @mock.patch.object(metadata_driver.LOG, 'error')
+    @mock.patch.object(driver_base.LOG, 'error')
     def test_spawn_metadata_proxy_handles_process_exception(self, error_log):
         process_instance = mock.Mock(active=False)
         process_instance.enable.side_effect = (
@@ -173,7 +187,9 @@ class TestMetadataDriverProcess(base.BaseTestCase):
 
         with mock.patch.object(metadata_driver.MetadataDriver,
                                '_get_metadata_proxy_process_manager',
-                               return_value=process_instance):
+                               return_value=process_instance),\
+                mock.patch.object(driver_base.MetadataDriverBase,
+                                  '_get_haproxy_configurator'):
             process_monitor = mock.Mock()
             network_id = 123456
 
@@ -190,22 +206,18 @@ class TestMetadataDriverProcess(base.BaseTestCase):
 
     def test_create_config_file_wrong_user(self):
         with mock.patch('pwd.getpwnam', side_effect=KeyError):
-            config = metadata_driver.HaproxyConfigurator(mock.ANY, mock.ANY,
-                                                         mock.ANY, mock.ANY,
-                                                         mock.ANY, self.EUNAME,
-                                                         self.EGNAME, mock.ANY,
-                                                         mock.ANY, mock.ANY)
             self.assertRaises(comm_meta.InvalidUserOrGroupException,
-                              config.create_config_file)
+                              metadata_driver.HaproxyConfigurator, mock.ANY,
+                              mock.ANY, mock.ANY, mock.ANY, mock.ANY,
+                              self.EUNAME, self.EGNAME, mock.ANY, mock.ANY,
+                              mock.ANY)
 
     def test_create_config_file_wrong_group(self):
         with mock.patch('grp.getgrnam', side_effect=KeyError),\
                 mock.patch('pwd.getpwnam',
                            return_value=test_utils.FakeUser(self.EUNAME)):
-            config = metadata_driver.HaproxyConfigurator(mock.ANY, mock.ANY,
-                                                         mock.ANY, mock.ANY,
-                                                         mock.ANY, self.EUNAME,
-                                                         self.EGNAME, mock.ANY,
-                                                         mock.ANY, mock.ANY)
             self.assertRaises(comm_meta.InvalidUserOrGroupException,
-                              config.create_config_file)
+                              metadata_driver.HaproxyConfigurator, mock.ANY,
+                              mock.ANY, mock.ANY, mock.ANY, mock.ANY,
+                              self.EUNAME, self.EGNAME, mock.ANY, mock.ANY,
+                              mock.ANY)
