@@ -12,31 +12,31 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from concurrent import futures
 import os.path
+import queue
 import random
 import re
 import sys
+import threading
+import time
 from unittest import mock
 
 import ddt
-import eventlet
-from eventlet import queue
 import netaddr
 from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import timeutils
 from osprofiler import profiler
-import testscenarios
 import testtools
 
 from neutron.common import utils
 from neutron.tests import base
 from neutron.tests.unit import tests
 
-load_tests = testscenarios.load_tests_apply_scenarios
 
-
-class _PortRange(object):
+class _PortRange:
     """A linked list of port ranges."""
     def __init__(self, base, prev_ref=None):
         self.base = base
@@ -75,12 +75,13 @@ class _PortRange(object):
         return [str(self)]
 
 
-_hex_str = lambda num: format(num, '#06x')
+def _hex_str(num):
+    return format(num, '#06x')
 
 
 def _hex_format(port, mask):
     if mask != 0xffff:
-        return "%s/%s" % (_hex_str(port), _hex_str(0xffff & mask))
+        return f"{_hex_str(port)}/{_hex_str(0xffff & mask)}"
     return _hex_str(port)
 
 
@@ -117,11 +118,14 @@ class TestExceptionLogger(base.BaseTestCase):
         logger = mock.Mock()
 
         @utils.exception_logger(logger=logger)
-        def func():
-            return result
+        def func(ret):
+            ret.append(result)
 
-        gt = eventlet.spawn(func)
-        self.assertEqual(result, gt.wait())
+        ret_value = []
+        _thread = threading.Thread(target=func, args=(ret_value, ))
+        _thread.start()
+        _thread.join()
+        self.assertEqual(result, ret_value[0])
         self.assertFalse(logger.called)
 
     def test_spawn_raise(self):
@@ -132,8 +136,8 @@ class TestExceptionLogger(base.BaseTestCase):
         def func():
             raise RuntimeError(result)
 
-        gt = eventlet.spawn(func)
-        self.assertRaises(RuntimeError, gt.wait)
+        _thread = threading.Thread(target=func)
+        _thread.start()
         self.assertTrue(logger.called)
 
     def test_pool_spawn_normal(self):
@@ -144,10 +148,9 @@ class TestExceptionLogger(base.BaseTestCase):
         def func(i):
             calls(i)
 
-        pool = eventlet.GreenPool(4)
-        for i in range(0, 4):
-            pool.spawn(func, i)
-        pool.waitall()
+        with futures.ThreadPoolExecutor(max_workers=4) as executor:
+            fs = [executor.submit(func, i) for i in range(4)]
+            all(f.done() for f in fs)
 
         calls.assert_has_calls([mock.call(0), mock.call(1),
                                 mock.call(2), mock.call(3)],
@@ -164,15 +167,24 @@ class TestExceptionLogger(base.BaseTestCase):
                 raise RuntimeError(2)
             calls(i)
 
-        pool = eventlet.GreenPool(4)
-        for i in range(0, 4):
-            pool.spawn(func, i)
-        pool.waitall()
+        with futures.ThreadPoolExecutor(max_workers=4) as executor:
+            fs = [executor.submit(func, i) for i in range(4)]
+            all(f.done() for f in fs)
 
         calls.assert_has_calls([mock.call(0), mock.call(1), mock.call(3)],
                                any_order=True)
         self.assertTrue(logger.called)
 
+    def test_wait_until_true(self):
+
+        class FalseException(Exception):
+            pass
+
+        self.assertRaises(
+            FalseException,
+            utils.wait_until_true,
+            lambda: False, timeout=1, exception=FalseException)
+
 
 class TestDvrServices(base.BaseTestCase):
 
@@ -410,13 +422,13 @@ class TestThrottler(base.BaseTestCase):
 
         throttled_func()
 
-        sleep = utils.eventlet.sleep
+        sleep = utils.time.sleep
 
         def sleep_mock(amount_to_sleep):
             sleep(amount_to_sleep)
             self.assertGreaterEqual(threshold, amount_to_sleep)
 
-        with mock.patch.object(utils.eventlet, "sleep",
+        with mock.patch.object(utils.time, "sleep",
                                side_effect=sleep_mock):
             throttled_func()
 
@@ -432,7 +444,7 @@ class TestThrottler(base.BaseTestCase):
         self.assertLess(timestamp, lock_with_timer.timestamp)
 
     def test_method_docstring_is_preserved(self):
-        class Klass(object):
+        class Klass:
             @utils.throttler()
             def method(self):
                 """Docstring"""
@@ -440,7 +452,7 @@ class TestThrottler(base.BaseTestCase):
         self.assertEqual("Docstring", Klass.method.__doc__)
 
     def test_method_still_callable(self):
-        class Klass(object):
+        class Klass:
             @utils.throttler()
             def method(self):
                 pass
@@ -449,7 +461,7 @@ class TestThrottler(base.BaseTestCase):
         obj.method()
 
 
-class BaseUnitConversionTest(object):
+class BaseUnitConversionTest:
 
     def test_bytes_to_bits(self):
         test_values = [
@@ -503,7 +515,7 @@ class TestIECUnitConversions(BaseUnitConversionTest, base.BaseTestCase):
 class TestRpBandwidthValidator(base.BaseTestCase):
 
     def setUp(self):
-        super(TestRpBandwidthValidator, self).setUp()
+        super().setUp()
         self.device_name_set = {'ens4', 'ens7'}
         self.valid_rp_bandwidths = {
             'ens7': {'egress': 10000, 'ingress': 10000}
@@ -524,13 +536,7 @@ class TestRpBandwidthValidator(base.BaseTestCase):
                           self.not_valid_rp_bandwidth, self.device_name_set)
 
 
-class SpawnWithOrWithoutProfilerTestCase(
-        testscenarios.WithScenarios, base.BaseTestCase):
-
-    scenarios = [
-        ('spawn', {'spawn_variant': utils.spawn}),
-        ('spawn_n', {'spawn_variant': utils.spawn_n}),
-    ]
+class SpawnWithOrWithoutProfilerTestCase(base.BaseTestCase):
 
     def _compare_profilers_in_parent_and_in_child(self, init_profiler):
 
@@ -546,14 +552,14 @@ class SpawnWithOrWithoutProfilerTestCase(
             if init_profiler:
                 profiler.init(hmac_key='fake secret')
 
-            self.spawn_variant(
-                lambda: q.put(is_profiler_initialized('in-child')))
+            utils.spawn_n(lambda: q.put(is_profiler_initialized('in-child')))
             q.put(is_profiler_initialized('in-parent'))
 
         # Make sure in parent we start with an uninitialized profiler by
-        # eventlet.spawn()-ing a new thread. Otherwise the unit test runner
-        # thread may leak an initialized profiler from one test to another.
-        eventlet.spawn(thread_with_no_leaked_profiler)
+        # spawning a new thread. Otherwise the unit test runner thread may
+        # leak an initialized profiler from one test to another.
+        _thread = threading.Thread(target=thread_with_no_leaked_profiler)
+        _thread.start()
 
         # In order to have some global protection against leaking initialized
         # profilers neutron.test.base.BaseTestCase.setup() also calls
@@ -577,13 +583,13 @@ class SpawnWithOrWithoutProfilerTestCase(
 
 
 @utils.SingletonDecorator
-class _TestSingletonClass1():
+class _TestSingletonClass1:
     def __init__(self, variable):
         self.variable = variable
 
 
 @utils.SingletonDecorator
-class _TestSingletonClass2():
+class _TestSingletonClass2:
     def __init__(self, variable):
         self.variable = variable
 
@@ -698,3 +704,84 @@ class ParsePermittedEthertypesTestCase(base.BaseTestCase):
                            '0x123R'),
                  ]
         mock_log.warning.assert_has_calls(calls)
+
+
+class StringMapTestCase(base.BaseTestCase):
+    data = {'a': 1, 'b': None, 'c': 'hi'}
+
+    def test_stringmap(self):
+        expected = {'a': '1', 'b': '', 'c': 'hi'}
+
+        self.assertEqual(expected, utils.stringmap(self.data))
+
+    def test_stringmap_custom_default(self):
+        self.assertEqual('None', utils.stringmap(self.data, 'None')['b'])
+
+
+class ThreadPoolExecutorWithBlockTestCase(base.BaseTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self._queue = queue.Queue()
+
+    def _check_values(self, expected_values):
+        # Retrieve all stored values
+        stored_values = []
+        while not self._queue.empty():
+            stored_values.append(self._queue.get())
+
+        for expected_value in expected_values:
+            self.assertIn(expected_value, stored_values)
+
+    def _add_tasks(self, tpool, num_tasks):
+        expected_values = []
+        for idx in range(num_tasks):
+            value1 = 'value1_%s' % idx
+            value2 = 'value2_%s' % idx
+            values = (value1, value2)
+            expected_values.append(values)
+            tpool.submit(self._task, self._queue, value1, value2=value2)
+        return expected_values
+
+    @staticmethod
+    def _task(_queue, value1, value2=None):
+        _queue.put((value1, value2))
+        time.sleep(1)
+
+    def test_simple_execution(self):
+        max_workers = 5
+        t1 = timeutils.utcnow()
+        with utils.ThreadPoolExecutorWithBlock(max_workers=max_workers) as \
+                tpool:
+            expected_values = self._add_tasks(tpool, max_workers * 3)
+
+        utils.wait_until_true(lambda: self._queue.qsize() == max_workers * 3,
+                              timeout=5, sleep=0.1)
+        t2 = timeutils.utcnow()
+        diff_seconds = (t2 - t1).total_seconds()
+        # NOTE(ralonsoh): we can't expect a 3 seconds sharp time gap, but at
+        # least the execution is contained in this interval. If each task takes
+        # 1 second, 5 threads can be executed in parallel and 15 workers are
+        # required, that will take at least 3 seconds and should complete in
+        # 6 seconds (2x).
+        self.assertTrue(3 < diff_seconds < 6)
+
+        self._check_values(expected_values)
+
+    def test_simple_execution_fast_exit(self):
+        max_workers = 5
+        t1 = timeutils.utcnow()
+        with utils.ThreadPoolExecutorWithBlock(max_workers=max_workers) as \
+                tpool:
+            expected_values = self._add_tasks(tpool, max_workers)
+            tpool.shutdown(wait=False)
+            self._add_tasks(tpool, max_workers)
+
+        utils.wait_until_true(lambda: self._queue.qsize() == max_workers,
+                              timeout=5, sleep=0.1)
+        t2 = timeutils.utcnow()
+        diff_seconds = (t2 - t1).total_seconds()
+        # Similar to above, execution will take at least 1 second and should
+        # complete in 2 seconds (2x).
+        self.assertTrue(1 < diff_seconds < 2)
+        self._check_values(expected_values)
