@@ -75,6 +75,16 @@ class SystemAdminTests(PortAPITestCase):
             base_policy.InvalidScope,
             policy.enforce, self.context, 'create_port', self.alt_target)
 
+    def test_create_port_with_device_id(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce, self.context, 'create_port:device_id',
+            self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce, self.context, 'create_port:device_id',
+            self.alt_target)
+
     def test_create_port_with_device_owner(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -283,6 +293,16 @@ class SystemAdminTests(PortAPITestCase):
             base_policy.InvalidScope,
             policy.enforce, self.context, 'update_port', self.alt_target)
 
+    def test_update_port_with_device_id(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce, self.context, 'update_port:device_id',
+            self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce, self.context, 'update_port:device_id',
+            self.alt_target)
+
     def test_update_port_with_device_owner(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -454,6 +474,14 @@ class AdminTests(PortAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'create_port', self.alt_target))
 
+    def test_create_port_with_device_id(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_port:device_id',
+                           self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'create_port:device_id',
+                           self.alt_target))
+
     def test_create_port_with_device_owner(self):
         target = self.target.copy()
         target['device_owner'] = 'network:test'
@@ -663,6 +691,14 @@ class AdminTests(PortAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'update_port', self.alt_target))
 
+    def test_update_port_with_device_id(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'update_port:device_id',
+                           self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'update_port:device_id',
+                           self.alt_target))
+
     def test_update_port_with_device_owner(self):
         target = self.target.copy()
         target['device_owner'] = 'network:test'
@@ -822,6 +858,15 @@ class ProjectManagerTests(AdminTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'create_port', self.alt_target)
 
+    def test_create_port_with_device_id(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_port:device_id',
+                           self.target))
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce, self.context, 'create_port:device_id',
+            self.alt_target)
+
     def test_create_port_with_device_owner(self):
         target = self.target.copy()
         target['device_owner'] = 'network:test'
@@ -1062,6 +1107,14 @@ class ProjectManagerTests(AdminTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'update_port', self.alt_target)
 
+    def test_update_port_with_device_id(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'update_port:device_id', self.target))
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce, self.context, 'update_port:device_id',
+            self.alt_target)
+
     def test_update_port_with_device_owner(self):
         target = self.target.copy()
         target['device_owner'] = 'network:test'
@@ -1245,8 +1298,7 @@ class ProjectMemberTests(ProjectManagerTests):
 
     def test_create_port_with_mac_address(self):
         self.assertTrue(
-            policy.enforce(self.context, 'create_port:mac_address',
-                           self.target))
+            policy.enforce(self.context, 'create_port:mac_address', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'create_port:mac_address',
@@ -1262,8 +1314,7 @@ class ProjectMemberTests(ProjectManagerTests):
 
     def test_create_port_with_fixed_ips_and_ip_address(self):
         self.assertTrue(
-            policy.enforce(self.context, 'create_port:fixed_ips:ip_address',
-                           self.target))
+            policy.enforce(self.context, 'create_port:fixed_ips:ip_address', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'create_port:fixed_ips:ip_address',
@@ -1271,8 +1322,7 @@ class ProjectMemberTests(ProjectManagerTests):
 
     def test_create_port_with_fixed_ips_and_subnet_id(self):
         self.assertTrue(
-            policy.enforce(self.context, 'create_port:fixed_ips:subnet_id',
-                           self.target))
+            policy.enforce(self.context, 'create_port:fixed_ips:subnet_id', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'create_port:fixed_ips:subnet_id',
@@ -1280,8 +1330,7 @@ class ProjectMemberTests(ProjectManagerTests):
 
     def test_create_port_with_port_security_enabled(self):
         self.assertTrue(
-            policy.enforce(self.context, 'create_port:port_security_enabled',
-                           self.target))
+            policy.enforce(self.context, 'create_port:port_security_enabled', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'create_port:port_security_enabled',
@@ -1290,8 +1339,7 @@ class ProjectMemberTests(ProjectManagerTests):
     def test_create_port_with_allowed_address_pairs(self):
         self.assertTrue(
             policy.enforce(
-                self.context, 'create_port:allowed_address_pairs',
-                self.target))
+                self.context, 'create_port:allowed_address_pairs', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
@@ -1301,8 +1349,7 @@ class ProjectMemberTests(ProjectManagerTests):
     def test_create_port_with_allowed_address_pairs_and_mac_address(self):
         self.assertTrue(
             policy.enforce(
-                self.context, 'create_port:allowed_address_pairs:mac_address',
-                self.target))
+                self.context, 'create_port:allowed_address_pairs:mac_address', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
@@ -1312,8 +1359,7 @@ class ProjectMemberTests(ProjectManagerTests):
     def test_create_port_with_allowed_address_pairs_and_ip_address(self):
         self.assertTrue(
             policy.enforce(
-                self.context, 'create_port:allowed_address_pairs:ip_address',
-                self.target))
+                self.context, 'create_port:allowed_address_pairs:ip_address', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
@@ -1368,9 +1414,7 @@ class ProjectMemberTests(ProjectManagerTests):
 
     def test_update_port_with_port_security_enabled(self):
         self.assertTrue(
-            policy.enforce(
-                self.context, 'update_port:port_security_enabled',
-                self.target))
+            policy.enforce(self.context, 'update_port:port_security_enabled', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'update_port:port_security_enabled',
@@ -1378,9 +1422,7 @@ class ProjectMemberTests(ProjectManagerTests):
 
     def test_update_port_with_allowed_address_pairs(self):
         self.assertTrue(
-            policy.enforce(
-                self.context, 'update_port:allowed_address_pairs',
-                self.target))
+            policy.enforce(self.context, 'update_port:allowed_address_pairs', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
@@ -1389,9 +1431,7 @@ class ProjectMemberTests(ProjectManagerTests):
 
     def test_update_port_with_allowed_address_pairs_and_mac_address(self):
         self.assertTrue(
-            policy.enforce(
-                self.context, 'update_port:allowed_address_pairs:mac_address',
-                self.target))
+            policy.enforce(self.context, 'update_port:allowed_address_pairs:mac_address', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
@@ -1400,9 +1440,7 @@ class ProjectMemberTests(ProjectManagerTests):
 
     def test_update_port_with_allowed_address_pairs_and_ip_address(self):
         self.assertTrue(
-            policy.enforce(
-                self.context, 'update_port:allowed_address_pairs:ip_address',
-                self.target))
+            policy.enforce(self.context, 'update_port:allowed_address_pairs:ip_address', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
@@ -1448,6 +1486,16 @@ class ProjectReaderTests(ProjectMemberTests):
             policy.enforce, self.context, 'create_port:mac_address',
             self.alt_target)
 
+    def test_create_port_with_device_id(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce, self.context, 'create_port:device_id',
+            self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce, self.context, 'create_port:device_id',
+            self.alt_target)
+
     def test_create_port_with_fixed_ips(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
@@ -1564,6 +1612,16 @@ class ProjectReaderTests(ProjectMemberTests):
             policy.enforce, self.context, 'update_port:device_owner',
             alt_target)
 
+    def test_update_port_with_device_id(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce, self.context, 'update_port:device_id',
+            self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce, self.context, 'update_port:device_id',
+            self.alt_target)
+
     def test_update_port_with_port_security_enabled(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
@@ -1683,6 +1741,14 @@ class ServiceRoleTests(PortAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'create_port', self.target))
 
+    def test_create_port_with_device_id(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'create_port:device_id',
+                           self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'create_port:device_id',
+                           self.alt_target))
+
     def test_create_port_with_device_owner(self):
         self.assertTrue(
             policy.enforce(
@@ -1730,25 +1796,22 @@ class ServiceRoleTests(PortAPITestCase):
                            'create_port:binding:vnic_type', self.target))
 
     def test_create_port_with_allowed_address_pairs(self):
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'create_port:allowed_address_pairs',
-            self.target)
+        self.assertTrue(
+            policy.enforce(
+                self.context, 'create_port:allowed_address_pairs',
+                self.target))
 
     def test_create_port_with_allowed_address_pairs_and_mac_address(self):
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'create_port:allowed_address_pairs:mac_address',
-            self.alt_target)
+        self.assertTrue(
+            policy.enforce(
+                self.context, 'create_port:allowed_address_pairs:mac_address',
+                self.alt_target))
 
     def test_create_port_with_allowed_address_pairs_and_ip_address(self):
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'create_port:allowed_address_pairs:ip_address',
-            self.target)
+        self.assertTrue(
+            policy.enforce(
+                self.context, 'create_port:allowed_address_pairs:ip_address',
+                self.target))
 
     def test_create_port_tags(self):
         self.assertRaises(
@@ -1791,6 +1854,14 @@ class ServiceRoleTests(PortAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'update_port', self.target))
 
+    def test_update_port_with_device_id(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'update_port:device_id',
+                           self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'update_port:device_id',
+                           self.alt_target))
+
     def test_update_port_with_device_owner(self):
         self.assertTrue(
             policy.enforce(
@@ -1838,25 +1909,22 @@ class ServiceRoleTests(PortAPITestCase):
                 self.context, 'update_port:binding:vnic_type', self.target))
 
     def test_update_port_with_allowed_address_pairs(self):
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'update_port:allowed_address_pairs',
-            self.target)
+        self.assertTrue(
+            policy.enforce(
+                self.context, 'update_port:allowed_address_pairs',
+                self.target))
 
     def test_update_port_with_allowed_address_pairs_and_mac_address(self):
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'update_port:allowed_address_pairs:mac_address',
-            self.target)
+        self.assertTrue(
+            policy.enforce(
+                self.context, 'update_port:allowed_address_pairs:mac_address',
+                self.target))
 
     def test_update_port_with_allowed_address_pairs_and_ip_address(self):
-        self.assertRaises(
-            base_policy.PolicyNotAuthorized,
-            policy.enforce,
-            self.context, 'update_port:allowed_address_pairs:ip_address',
-            self.target)
+        self.assertTrue(
+            policy.enforce(
+                self.context, 'update_port:allowed_address_pairs:ip_address',
+                self.target))
 
     def test_update_port_data_plane_status(self):
         self.assertRaises(
