@@ -44,13 +44,13 @@ class SystemAdminQosPolicyTests(QosPolicyAPITestCase):
             base_policy.InvalidScope,
             policy.enforce, self.context, 'get_policy', self.alt_target)
 
-    def test_get_policies_tags(self):
+    def test_get_policy_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
-            policy.enforce, self.context, 'get_policies_tags', self.target)
+            policy.enforce, self.context, 'get_policy:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
-            policy.enforce, self.context, 'get_policies_tags', self.alt_target)
+            policy.enforce, self.context, 'get_policy:tags', self.alt_target)
 
     def test_create_policy(self):
         self.assertRaises(
@@ -60,13 +60,13 @@ class SystemAdminQosPolicyTests(QosPolicyAPITestCase):
             base_policy.InvalidScope,
             policy.enforce, self.context, 'create_policy', self.alt_target)
 
-    def test_create_policies_tags(self):
+    def test_create_policy_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
-            policy.enforce, self.context, 'create_policies_tags', self.target)
+            policy.enforce, self.context, 'create_policy:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
-            policy.enforce, self.context, 'create_policies_tags',
+            policy.enforce, self.context, 'create_policy:tags',
             self.alt_target)
 
     def test_update_policy(self):
@@ -77,13 +77,13 @@ class SystemAdminQosPolicyTests(QosPolicyAPITestCase):
             base_policy.InvalidScope,
             policy.enforce, self.context, 'update_policy', self.alt_target)
 
-    def test_update_policies_tags(self):
+    def test_update_policy_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
-            policy.enforce, self.context, 'update_policies_tags', self.target)
+            policy.enforce, self.context, 'update_policy:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
-            policy.enforce, self.context, 'update_policies_tags',
+            policy.enforce, self.context, 'update_policy:tags',
             self.alt_target)
 
     def test_delete_policy(self):
@@ -94,13 +94,13 @@ class SystemAdminQosPolicyTests(QosPolicyAPITestCase):
             base_policy.InvalidScope,
             policy.enforce, self.context, 'delete_policy', self.alt_target)
 
-    def test_delete_policies_tags(self):
+    def test_delete_policy_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
-            policy.enforce, self.context, 'delete_policies_tags', self.target)
+            policy.enforce, self.context, 'delete_policy:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
-            policy.enforce, self.context, 'delete_policies_tags',
+            policy.enforce, self.context, 'delete_policy:tags',
             self.alt_target)
 
 
@@ -130,11 +130,11 @@ class AdminQosPolicyTests(QosPolicyAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'get_policy', self.alt_target))
 
-    def test_get_policies_tags(self):
+    def test_get_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'get_policies_tags', self.target))
+            policy.enforce(self.context, 'get_policy:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'get_policies_tags', self.alt_target))
+            policy.enforce(self.context, 'get_policy:tags', self.alt_target))
 
     def test_create_policy(self):
         self.assertTrue(
@@ -142,11 +142,11 @@ class AdminQosPolicyTests(QosPolicyAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'create_policy', self.alt_target))
 
-    def test_create_policies_tags(self):
+    def test_create_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'create_policies_tags', self.target))
+            policy.enforce(self.context, 'create_policy:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'create_policies_tags',
+            policy.enforce(self.context, 'create_policy:tags',
                            self.alt_target))
 
     def test_update_policy(self):
@@ -155,11 +155,11 @@ class AdminQosPolicyTests(QosPolicyAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'update_policy', self.alt_target))
 
-    def test_update_policies_tags(self):
+    def test_update_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'update_policies_tags', self.target))
+            policy.enforce(self.context, 'update_policy:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'update_policies_tags',
+            policy.enforce(self.context, 'update_policy:tags',
                            self.alt_target))
 
     def test_delete_policy(self):
@@ -168,11 +168,11 @@ class AdminQosPolicyTests(QosPolicyAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'delete_policy', self.alt_target))
 
-    def test_delete_policies_tags(self):
+    def test_delete_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'delete_policies_tags', self.target))
+            policy.enforce(self.context, 'delete_policy:tags', self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'delete_policies_tags',
+            policy.enforce(self.context, 'delete_policy:tags',
                            self.alt_target))
 
 
@@ -189,12 +189,12 @@ class ProjectManagerQosPolicyTests(AdminQosPolicyTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'get_policy', self.alt_target)
 
-    def test_get_policies_tags(self):
+    def test_get_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'get_policies_tags', self.target))
+            policy.enforce(self.context, 'get_policy:tags', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'get_policies_tags',
+            policy.enforce, self.context, 'get_policy:tags',
             self.alt_target)
 
     def test_create_policy(self):
@@ -204,12 +204,12 @@ class ProjectManagerQosPolicyTests(AdminQosPolicyTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'create_policy', self.alt_target)
 
-    def test_create_policies_tags(self):
+    def test_create_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'create_policies_tags', self.target))
+            policy.enforce(self.context, 'create_policy:tags', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'create_policies_tags',
+            policy.enforce, self.context, 'create_policy:tags',
             self.alt_target)
 
     def test_update_policy(self):
@@ -219,12 +219,12 @@ class ProjectManagerQosPolicyTests(AdminQosPolicyTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'update_policy', self.alt_target)
 
-    def test_update_policies_tags(self):
+    def test_update_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'update_policies_tags', self.target))
+            policy.enforce(self.context, 'update_policy:tags', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'update_policies_tags',
+            policy.enforce, self.context, 'update_policy:tags',
             self.alt_target)
 
     def test_delete_policy(self):
@@ -234,12 +234,12 @@ class ProjectManagerQosPolicyTests(AdminQosPolicyTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'delete_policy', self.alt_target)
 
-    def test_delete_policies_tags(self):
+    def test_delete_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'delete_policies_tags', self.target))
+            policy.enforce(self.context, 'delete_policy:tags', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'delete_policies_tags',
+            policy.enforce, self.context, 'delete_policy:tags',
             self.alt_target)
 
 
@@ -256,12 +256,12 @@ class ProjectMemberQosPolicyTests(ProjectManagerQosPolicyTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'get_policy', self.alt_target)
 
-    def test_get_policies_tags(self):
+    def test_get_policy_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'get_policies_tags', self.target))
+            policy.enforce(self.context, 'get_policy:tags', self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'get_policies_tags',
+            policy.enforce, self.context, 'get_policy:tags',
             self.alt_target)
 
     def test_create_policy(self):
@@ -272,13 +272,13 @@ class ProjectMemberQosPolicyTests(ProjectManagerQosPolicyTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'create_policy', self.alt_target)
 
-    def test_create_policies_tags(self):
+    def test_create_policy_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'create_policies_tags', self.target)
+            policy.enforce, self.context, 'create_policy:tags', self.target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'create_policies_tags',
+            policy.enforce, self.context, 'create_policy:tags',
             self.alt_target)
 
     def test_update_policy(self):
@@ -289,13 +289,13 @@ class ProjectMemberQosPolicyTests(ProjectManagerQosPolicyTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'update_policy', self.alt_target)
 
-    def test_update_policies_tags(self):
+    def test_update_policy_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'update_policies_tags', self.target)
+            policy.enforce, self.context, 'update_policy:tags', self.target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'update_policies_tags',
+            policy.enforce, self.context, 'update_policy:tags',
             self.alt_target)
 
     def test_delete_policy(self):
@@ -306,13 +306,13 @@ class ProjectMemberQosPolicyTests(ProjectManagerQosPolicyTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'delete_policy', self.alt_target)
 
-    def test_delete_policies_tags(self):
+    def test_delete_policy_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'delete_policies_tags', self.target)
+            policy.enforce, self.context, 'delete_policy:tags', self.target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'delete_policies_tags',
+            policy.enforce, self.context, 'delete_policy:tags',
             self.alt_target)
 
 
@@ -334,40 +334,40 @@ class ServiceRoleQosPolicyTests(QosPolicyAPITestCase):
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'get_policy', self.target)
 
-    def test_get_policies_tags(self):
+    def test_get_policy_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'get_policies_tags', self.target)
+            policy.enforce, self.context, 'get_policy:tags', self.target)
 
     def test_create_policy(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'create_policy', self.target)
 
-    def test_create_policies_tags(self):
+    def test_create_policy_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'create_policies_tags', self.target)
+            policy.enforce, self.context, 'create_policy:tags', self.target)
 
     def test_update_policy(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'update_policy', self.target)
 
-    def test_update_policies_tags(self):
+    def test_update_policy_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'update_policies_tags', self.target)
+            policy.enforce, self.context, 'update_policy:tags', self.target)
 
     def test_delete_policy(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce, self.context, 'delete_policy', self.target)
 
-    def test_delete_policies_tags(self):
+    def test_delete_policy_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'delete_policies_tags', self.target)
+            policy.enforce, self.context, 'delete_policy:tags', self.target)
 
 
 class QosRuleTypeAPITestCase(base.PolicyBaseTestCase):
