@@ -47,15 +47,15 @@ class SystemAdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce,
             self.context, 'create_security_group', self.alt_target)
 
-    def test_create_security_groups_tags(self):
+    def test_create_security_group_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'create_security_groups_tags', self.target)
+            self.context, 'create_security_group:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'create_security_groups_tags', self.alt_target)
+            self.context, 'create_security_group:tags', self.alt_target)
 
     def test_get_security_group(self):
         self.assertRaises(
@@ -67,15 +67,15 @@ class SystemAdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce,
             self.context, 'get_security_group', self.alt_target)
 
-    def test_get_security_groups_tags(self):
+    def test_get_security_group_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'get_security_groups_tags', self.target)
+            self.context, 'get_security_group:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'get_security_groups_tags', self.alt_target)
+            self.context, 'get_security_group:tags', self.alt_target)
 
     def test_update_security_group(self):
         self.assertRaises(
@@ -87,15 +87,15 @@ class SystemAdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce,
             self.context, 'update_security_group', self.alt_target)
 
-    def test_update_security_groups_tags(self):
+    def test_update_security_group_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'update_security_groups_tags', self.target)
+            self.context, 'update_security_group:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'update_security_groups_tags', self.alt_target)
+            self.context, 'update_security_group:tags', self.alt_target)
 
     def test_delete_security_group(self):
         self.assertRaises(
@@ -107,15 +107,15 @@ class SystemAdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce,
             self.context, 'delete_security_group', self.alt_target)
 
-    def test_delete_security_groups_tags(self):
+    def test_delete_security_group_tags(self):
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'delete_security_groups_tags', self.target)
+            self.context, 'delete_security_group:tags', self.target)
         self.assertRaises(
             base_policy.InvalidScope,
             policy.enforce,
-            self.context, 'delete_security_groups_tags', self.alt_target)
+            self.context, 'delete_security_group:tags', self.alt_target)
 
 
 class SystemMemberSecurityGroupTests(SystemAdminSecurityGroupTests):
@@ -145,12 +145,12 @@ class AdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce(
                 self.context, 'create_security_group', self.alt_target))
 
-    def test_create_security_groups_tags(self):
+    def test_create_security_group_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'create_security_groups_tags',
+            policy.enforce(self.context, 'create_security_group:tags',
                            self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'create_security_groups_tags',
+            policy.enforce(self.context, 'create_security_group:tags',
                            self.alt_target))
 
     def test_get_security_group(self):
@@ -160,12 +160,12 @@ class AdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce(
                 self.context, 'get_security_group', self.alt_target))
 
-    def test_get_security_groups_tags(self):
+    def test_get_security_group_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'get_security_groups_tags',
+            policy.enforce(self.context, 'get_security_group:tags',
                            self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'get_security_groups_tags',
+            policy.enforce(self.context, 'get_security_group:tags',
                            self.alt_target))
 
     def test_update_security_group(self):
@@ -175,12 +175,12 @@ class AdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce(
                 self.context, 'update_security_group', self.alt_target))
 
-    def test_update_security_groups_tags(self):
+    def test_update_security_group_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'update_security_groups_tags',
+            policy.enforce(self.context, 'update_security_group:tags',
                            self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'update_security_groups_tags',
+            policy.enforce(self.context, 'update_security_group:tags',
                            self.alt_target))
 
     def test_delete_security_group(self):
@@ -190,12 +190,12 @@ class AdminSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce(
                 self.context, 'delete_security_group', self.alt_target))
 
-    def test_delete_security_groups_tags(self):
+    def test_delete_security_group_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'delete_security_groups_tags',
+            policy.enforce(self.context, 'delete_security_group:tags',
                            self.target))
         self.assertTrue(
-            policy.enforce(self.context, 'delete_security_groups_tags',
+            policy.enforce(self.context, 'delete_security_group:tags',
                            self.alt_target))
 
 
@@ -213,14 +213,14 @@ class ProjectManagerSecurityGroupTests(AdminSecurityGroupTests):
             policy.enforce,
             self.context, 'create_security_group', self.alt_target)
 
-    def test_create_security_groups_tags(self):
+    def test_create_security_group_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'create_security_groups_tags',
+            policy.enforce(self.context, 'create_security_group:tags',
                            self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'create_security_groups_tags', self.alt_target)
+            self.context, 'create_security_group:tags', self.alt_target)
 
     def test_get_security_group(self):
         self.assertTrue(
@@ -230,13 +230,13 @@ class ProjectManagerSecurityGroupTests(AdminSecurityGroupTests):
             policy.enforce,
             self.context, 'get_security_group', self.alt_target)
 
-    def test_get_security_groups_tags(self):
+    def test_get_security_group_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'get_security_groups_tags',
+            policy.enforce(self.context, 'get_security_group:tags',
                            self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized, policy.enforce,
-            self.context, 'get_security_groups_tags', self.alt_target)
+            self.context, 'get_security_group:tags', self.alt_target)
 
     def test_update_security_group(self):
         self.assertTrue(
@@ -246,13 +246,13 @@ class ProjectManagerSecurityGroupTests(AdminSecurityGroupTests):
             policy.enforce,
             self.context, 'update_security_group', self.alt_target)
 
-    def test_update_security_groups_tags(self):
+    def test_update_security_group_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'update_security_groups_tags',
+            policy.enforce(self.context, 'update_security_group:tags',
                            self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
-            policy.enforce, self.context, 'update_security_groups_tags',
+            policy.enforce, self.context, 'update_security_group:tags',
             self.alt_target)
 
     def test_delete_security_group(self):
@@ -263,13 +263,13 @@ class ProjectManagerSecurityGroupTests(AdminSecurityGroupTests):
             policy.enforce,
             self.context, 'delete_security_group', self.alt_target)
 
-    def test_delete_security_groups_tags(self):
+    def test_delete_security_group_tags(self):
         self.assertTrue(
-            policy.enforce(self.context, 'delete_security_groups_tags',
+            policy.enforce(self.context, 'delete_security_group:tags',
                            self.target))
         self.assertRaises(
             base_policy.PolicyNotAuthorized, policy.enforce,
-            self.context, 'delete_security_groups_tags', self.alt_target)
+            self.context, 'delete_security_group:tags', self.alt_target)
 
 
 class ProjectMemberSecurityGroupTests(ProjectManagerSecurityGroupTests):
@@ -295,15 +295,15 @@ class ProjectReaderSecurityGroupTests(ProjectMemberSecurityGroupTests):
             policy.enforce,
             self.context, 'create_security_group', self.alt_target)
 
-    def test_create_security_groups_tags(self):
+    def test_create_security_group_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'create_security_groups_tags', self.target)
+            self.context, 'create_security_group:tags', self.target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'create_security_groups_tags', self.alt_target)
+            self.context, 'create_security_group:tags', self.alt_target)
 
     def test_update_security_group(self):
         self.assertRaises(
@@ -315,15 +315,15 @@ class ProjectReaderSecurityGroupTests(ProjectMemberSecurityGroupTests):
             policy.enforce,
             self.context, 'update_security_group', self.alt_target)
 
-    def test_update_security_groups_tags(self):
+    def test_update_security_group_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'update_security_groups_tags', self.target)
+            self.context, 'update_security_group:tags', self.target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'update_security_groups_tags', self.alt_target)
+            self.context, 'update_security_group:tags', self.alt_target)
 
     def test_delete_security_group(self):
         self.assertRaises(
@@ -335,15 +335,15 @@ class ProjectReaderSecurityGroupTests(ProjectMemberSecurityGroupTests):
             policy.enforce,
             self.context, 'delete_security_group', self.alt_target)
 
-    def test_delete_security_groups_tags(self):
+    def test_delete_security_group_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'delete_security_groups_tags', self.target)
+            self.context, 'delete_security_group:tags', self.target)
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'delete_security_groups_tags', self.alt_target)
+            self.context, 'delete_security_group:tags', self.alt_target)
 
 
 class ServiceRoleSecurityGroupTests(SecurityGroupAPITestCase):
@@ -358,11 +358,11 @@ class ServiceRoleSecurityGroupTests(SecurityGroupAPITestCase):
             policy.enforce,
             self.context, 'create_security_group', self.target)
 
-    def test_create_security_groups_tags(self):
+    def test_create_security_group_tags(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
             policy.enforce,
-            self.context, 'create_security_groups_tags', self.target)
+            self.context, 'create_security_group:tags', self.target)
 
     def test_get_security_group(self):
         self.assertRaises(
