@@ -17,11 +17,12 @@ import copy
 
 from neutron_lib.api.definitions import extra_dhcp_opt as edo_ext
 from neutron_lib import constants
+from neutron_lib.db import api as db_api
 import webob.exc
 
 from neutron.db import db_base_plugin_v2
 from neutron.db import extradhcpopt_db as edo_db
-from neutron.tests.unit.db import test_db_base_plugin_v2
+from neutron.tests.common import test_db_base_plugin_v2
 
 
 DB_PLUGIN_KLASS = (
@@ -34,19 +35,19 @@ class ExtraDhcpOptTestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
     associating ports with extra dhcp options.
     """
 
-    supported_extension_aliases = ["extra_dhcp_opt"]
+    supported_extension_aliases = [edo_ext.ALIAS]
 
     def create_port(self, context, port):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             edos = port['port'].get(edo_ext.EXTRADHCPOPTS, [])
-            new_port = super(ExtraDhcpOptTestPlugin, self).create_port(
+            new_port = super().create_port(
                 context, port)
             self._process_port_create_extra_dhcp_opts(context, new_port, edos)
         return new_port
 
     def update_port(self, context, id, port):
-        with context.session.begin(subtransactions=True):
-            rtn_port = super(ExtraDhcpOptTestPlugin, self).update_port(
+        with db_api.CONTEXT_WRITER.using(context):
+            rtn_port = super().update_port(
                 context, id, port)
             self._update_extra_dhcp_opts_on_port(context, id, port, rtn_port)
         return rtn_port
@@ -55,7 +56,7 @@ class ExtraDhcpOptTestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
 class ExtraDhcpOptDBTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
 
     def setUp(self, plugin=DB_PLUGIN_KLASS):
-        super(ExtraDhcpOptDBTestCase, self).setUp(plugin=plugin)
+        super().setUp(plugin=plugin)
 
 
 class TestExtraDhcpOpt(ExtraDhcpOptDBTestCase):
