@@ -26,7 +26,7 @@ from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils as ovn_utils
 
 
-class FakeOvsdbNbOvnIdl(object):
+class FakeOvsdbNbOvnIdl:
 
     def __init__(self, **kwargs):
         self.lswitch_table = FakeOvsdbTable.create_one_ovsdb_table()
@@ -114,6 +114,8 @@ class FakeOvsdbNbOvnIdl(object):
         self.ls_set_dns_records = mock.Mock()
         self.get_floatingip = mock.Mock()
         self.get_floatingip.return_value = None
+        self.get_floatingips = mock.Mock()
+        self.get_floatingips.return_value = []
         self.check_revision_number = mock.Mock()
         self.lookup = mock.MagicMock()
         self.get_router_floatingip_lbs = mock.Mock()
@@ -130,6 +132,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.pg_acl_del = mock.Mock()
         self.pg_del = mock.Mock()
         self.pg_add = mock.Mock()
+        self.pg_get = mock.Mock()
         self.get_port_group = mock.Mock()
         self.pg_add_ports = mock.Mock()
         self.pg_del_ports = mock.Mock()
@@ -171,7 +174,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.schedule_new_gateway = mock.Mock()
 
 
-class FakeOvsdbSbOvnIdl(object):
+class FakeOvsdbSbOvnIdl:
 
     def __init__(self, **kwargs):
         self.chassis_exists = mock.Mock()
@@ -199,19 +202,19 @@ class FakeOvsdbSbOvnIdl(object):
         self.get_chassis_host_for_port = mock.Mock(return_value=set())
 
 
-class FakeOvsdbTransaction(object):
+class FakeOvsdbTransaction:
     def __init__(self, **kwargs):
         self.insert = mock.Mock()
 
 
-class FakePlugin(object):
+class FakePlugin:
 
     def __init__(self, **kwargs):
         self.get_ports = mock.Mock()
         self._get_port_security_group_bindings = mock.Mock()
 
 
-class FakeStandardAttribute(object):
+class FakeStandardAttribute:
 
     def __init__(self, _id=1, resource_type=mock.ANY, description=mock.ANY,
                  revision_number=1):
@@ -221,14 +224,14 @@ class FakeStandardAttribute(object):
         self.revision_number = revision_number
 
 
-class FakeQosNetworkPolicyBinding(object):
+class FakeQosNetworkPolicyBinding:
 
     def __init__(self, policy_id=mock.ANY, network_id=mock.ANY):
         self.policy_id = policy_id
         self.network_id = network_id
 
 
-class FakeQosFIPPolicyBinding(object):
+class FakeQosFIPPolicyBinding:
 
     def __init__(self, policy_id=mock.ANY, fip_id=mock.ANY):
         self.policy_id = policy_id
@@ -250,7 +253,7 @@ class FakeResource(dict):
             A dictionary with all methods
         """
         info = info or {}
-        super(FakeResource, self).__init__(info)
+        super().__init__(info)
         methods = methods or {}
 
         self.__name__ = type(self).__name__
@@ -285,8 +288,8 @@ class FakeResource(dict):
     def __repr__(self):
         reprkeys = sorted(k for k in self.__dict__.keys() if k[0] != '_' and
                           k != 'manager')
-        info = ", ".join("%s=%s" % (k, getattr(self, k)) for k in reprkeys)
-        return "<%s %s>" % (self.__class__.__name__, info)
+        info = ", ".join(f"{k}={getattr(self, k)}" for k in reprkeys)
+        return f"<{self.__class__.__name__} {info}>"
 
     def keys(self):
         return self._info.keys()
@@ -295,11 +298,11 @@ class FakeResource(dict):
         return self._info
 
     def update(self, info):
-        super(FakeResource, self).update(info)
+        super().update(info)
         self._add_details(info)
 
 
-class FakeNetwork(object):
+class FakeNetwork:
     """Fake one or more networks."""
 
     @staticmethod
@@ -341,7 +344,7 @@ class FakeNetwork(object):
                             loaded=True)
 
 
-class FakeNetworkContext(object):
+class FakeNetworkContext:
     def __init__(self, network, segments):
         self.fake_network = network
         self.fake_segments = segments
@@ -364,7 +367,7 @@ class FakeNetworkContext(object):
         return self.fake_segments
 
 
-class FakeSubnetContext(object):
+class FakeSubnetContext:
     def __init__(self, subnet, original_subnet=None, network=None):
         self.fake_subnet = subnet
         self.fake_original_subnet = original_subnet
@@ -485,7 +488,7 @@ class FakeOvsdbTable(FakeResource):
         return result
 
 
-class FakePort(object):
+class FakePort:
     """Fake one or more ports."""
 
     @staticmethod
@@ -533,7 +536,7 @@ class FakePort(object):
                             loaded=True)
 
 
-class FakePortContext(object):
+class FakePortContext:
     def __init__(self, port, host, segments_to_bind):
         self.fake_port = port
         self.fake_host = host
@@ -553,7 +556,7 @@ class FakePortContext(object):
         return self.fake_segments_to_bind
 
 
-class FakeSecurityGroup(object):
+class FakeSecurityGroup:
     """Fake one or more security groups."""
 
     @staticmethod
@@ -584,7 +587,7 @@ class FakeSecurityGroup(object):
                             loaded=True)
 
 
-class FakeSecurityGroupRule(object):
+class FakeSecurityGroupRule:
     """Fake one or more security group rules."""
 
     @staticmethod
@@ -608,8 +611,8 @@ class FakeSecurityGroupRule(object):
             'port_range_min': 22,
             'protocol': 'tcp',
             'remote_group_id': None,
-            'remote_ip_prefix': '0.0.0.0/0',
-            'normalized_cidr': '0.0.0.0/0',
+            'remote_ip_prefix': n_const.IPv4_ANY,
+            'normalized_cidr': n_const.IPv4_ANY,
             'security_group_id': 'security-group-id-' + fake_uuid,
             'tenant_id': 'project-id-' + fake_uuid,
         }
@@ -628,7 +631,7 @@ class FakeSecurityGroupRule(object):
                             loaded=True)
 
 
-class FakeSegment(object):
+class FakeSegment:
     """Fake one or more segments."""
 
     @staticmethod
@@ -658,7 +661,7 @@ class FakeSegment(object):
                             loaded=True)
 
 
-class FakeSubnet(object):
+class FakeSubnet:
     """Fake one or more subnets."""
 
     @staticmethod
@@ -698,7 +701,7 @@ class FakeSubnet(object):
                             loaded=True)
 
 
-class FakeFloatingIp(object):
+class FakeFloatingIp:
     """Fake one or more floating ips."""
 
     @staticmethod
@@ -744,7 +747,7 @@ class FakeFloatingIp(object):
                             loaded=True)
 
 
-class FakeRouter(object):
+class FakeRouter:
     """Fake one or more Neutron routers."""
 
     @staticmethod
@@ -781,7 +784,7 @@ class FakeRouter(object):
         return FakeResource(info=copy.deepcopy(router_attrs), loaded=True)
 
 
-class FakeOVNPort(object):
+class FakeOVNPort:
     """Fake one or more ports."""
 
     @staticmethod
@@ -842,7 +845,7 @@ FakeStaticRoute = collections.namedtuple(
     defaults=([],))
 
 
-class FakeOVNRouterPort(object):
+class FakeOVNRouterPort:
     """Fake one or more Logical Router Ports."""
 
     @staticmethod
@@ -878,7 +881,7 @@ class FakeOVNRouterPort(object):
         return type('Logical_Router_Port', (object, ), port_attrs)()
 
 
-class FakeOVNRouter(object):
+class FakeOVNRouter:
 
     @staticmethod
     def create_one_router(attrs=None):
@@ -908,7 +911,6 @@ class FakeOVNRouter(object):
             return subnet_id
 
         external_ids = {
-            ovn_const.OVN_GW_PORT_EXT_ID_KEY: router.get('gw_port_id') or '',
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
                 router.get('name', 'no_router_name')}
 
@@ -926,7 +928,7 @@ class FakeOVNRouter(object):
                 ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
                 ovn_const.OVN_SUBNET_EXT_ID_KEY: _get_subnet_id(gw_info)}
             routes.append(FakeStaticRoute(
-                ip_prefix='0.0.0.0/0', nexthop='',
+                ip_prefix=n_const.IPv4_ANY, nexthop='',
                 external_ids=external_ids))
             ports.append(FakeOVNRouterPort.create_one_port(
                 attrs={
@@ -941,7 +943,7 @@ class FakeOVNRouter(object):
              'static_routes': routes})
 
 
-class FakeChassis(object):
+class FakeChassis:
 
     @staticmethod
     def create(attrs=None, az_list=None, chassis_as_gw=False,
@@ -951,8 +953,8 @@ class FakeChassis(object):
                datapath_type=None):
         cms_opts = []
         if az_list:
-            cms_opts.append("%s=%s" % (ovn_const.CMS_OPT_AVAILABILITY_ZONES,
-                                       ':'.join(az_list)))
+            cms_opts.append("{}={}".format(
+                ovn_const.CMS_OPT_AVAILABILITY_ZONES, ':'.join(az_list)))
         if chassis_as_gw:
             cms_opts.append(ovn_const.CMS_OPT_CHASSIS_AS_GW)
 
@@ -960,28 +962,28 @@ class FakeChassis(object):
             cms_opts.append(ovn_const.CMS_OPT_CHASSIS_AS_EXTPORT_HOST)
 
         if rp_bandwidths:
-            cms_opts.append('%s=%s' % (n_const.RP_BANDWIDTHS,
-                                       ';'.join(rp_bandwidths)))
+            cms_opts.append('{}={}'.format(n_const.RP_BANDWIDTHS,
+                                           ';'.join(rp_bandwidths)))
         elif rp_bandwidths == '':  # Test wrongly defined parameter
             cms_opts.append('%s=' % n_const.RP_BANDWIDTHS)
 
         if rp_inventory_defaults:
-            inv_defaults = ';'.join('%s:%s' % (key, value) for key, value in
-                                    rp_inventory_defaults.items())
-            cms_opts.append('%s=%s' % (n_const.RP_INVENTORY_DEFAULTS,
-                                       inv_defaults))
+            inv_defaults = ';'.join('{}:{}'.format(
+                key, value) for key, value in rp_inventory_defaults.items())
+            cms_opts.append('{}={}'.format(n_const.RP_INVENTORY_DEFAULTS,
+                                           inv_defaults))
         elif rp_inventory_defaults == '':  # Test wrongly defined parameter
             cms_opts.append('%s=' % n_const.RP_INVENTORY_DEFAULTS)
 
         if rp_hypervisors:
-            cms_opts.append('%s=%s' % (n_const.RP_HYPERVISORS,
-                                       ';'.join(rp_hypervisors)))
+            cms_opts.append('{}={}'.format(n_const.RP_HYPERVISORS,
+                                           ';'.join(rp_hypervisors)))
         elif rp_hypervisors == '':  # Test wrongly defined parameter
             cms_opts.append('%s=' % n_const.RP_HYPERVISORS)
 
         if card_serial_number:
-            cms_opts.append('%s=%s' % (ovn_const.CMS_OPT_CARD_SERIAL_NUMBER,
-                                       card_serial_number))
+            cms_opts.append('{}={}'.format(
+                ovn_const.CMS_OPT_CARD_SERIAL_NUMBER, card_serial_number))
 
         # NOTE(ralonsoh): LP#1990229, once min OVN version >= 20.06, the CMS
         # options and the bridge mappings should be stored only in
