@@ -14,12 +14,12 @@
 from unittest import mock
 
 from neutron_lib.callbacks import events
+from neutron_lib.callbacks import priority_group
 from neutron_lib import fixture
 
 from neutron.plugins.ml2.drivers.openvswitch.agent import ovs_capabilities
 from neutron.services.trunk.drivers.openvswitch.agent import driver
 from neutron.tests import base
-from neutron.tests import tools
 from neutron_lib import constants
 
 
@@ -33,6 +33,6 @@ class CapabilitiesTest(base.BaseTestCase):
 
     def test_register(self):
         ovs_capabilities.register()
-        args = tools.get_subscribe_args(
-            driver.init_handler, constants.AGENT_TYPE_OVS, events.AFTER_INIT)
-        self._mgr.subscribe.assert_called_with(*args)
+        self._mgr.subscribe.assert_called_with(
+            driver.init_handler, constants.AGENT_TYPE_OVS, events.AFTER_INIT,
+            priority_group.PRIORITY_DEFAULT, False)
