@@ -1241,8 +1241,7 @@ class TestOvnSbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
                                return_value=hosts_in_neutron) as mock_ghmws:
             ovn_sb_synchronizer.sync_hostname_and_physical_networks(mock.ANY)
             mock_ghmws.assert_called_once_with(
-                mock.ANY,
-                include_agent_types=set(ovn_const.OVN_CONTROLLER_TYPES))
+                mock.ANY, include_agent_types=set(ovn_const.OVN_CONTROLLER_TYPES))
             all_hosts = set(hostname_with_physnets.keys()) | hosts_in_neutron
             self.assertEqual(
                 len(all_hosts),
