@@ -35,9 +35,9 @@ class ExpectedException(testtools.ExpectedException):
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
-        if super(ExpectedException, self).__exit__(exc_type,
-                                                   exc_value,
-                                                   traceback):
+        if super().__exit__(exc_type,
+                            exc_value,
+                            traceback):
             self.exception = exc_value
             return True
         return False
@@ -47,7 +47,7 @@ def create_request(path, body, content_type, method='GET',
                    query_string=None, context=None, headers=None):
     headers = headers or {}
     if query_string:
-        url = "%s?%s" % (path, query_string)
+        url = f"{path}?{query_string}"
     else:
         url = path
     req = wsgi.Request.blank(url)
@@ -76,14 +76,13 @@ class StaticSqlFixtureNoSchema(lib_fixtures.SqlFixture):
     def _init_resources(cls):
         if cls._GLOBAL_RESOURCES:
             return
-        else:
-            cls._GLOBAL_RESOURCES = True
-            cls.database_resource = provision.DatabaseResource(
-                "sqlite", db_api.get_context_manager())
-            dependency_resources = {}
-            for name, resource in cls.database_resource.resources:
-                dependency_resources[name] = resource.getResource()
-            cls.engine = dependency_resources['backend'].engine
+        cls._GLOBAL_RESOURCES = True
+        cls.database_resource = provision.DatabaseResource(
+            "sqlite", db_api.get_context_manager())
+        dependency_resources = {}
+        for name, resource in cls.database_resource.resources:
+            dependency_resources[name] = resource.getResource()
+        cls.engine = dependency_resources['backend'].engine
 
     def _delete_from_schema(self, engine):
         pass
@@ -100,7 +99,7 @@ class OpportunisticSqlFixture(lib_fixtures.SqlFixture):
     DRIVER = 'sqlite'
 
     def __init__(self, test):
-        super(OpportunisticSqlFixture, self).__init__()
+        super().__init__()
         self.test = test
 
     @classmethod
@@ -117,7 +116,7 @@ class OpportunisticSqlFixture(lib_fixtures.SqlFixture):
 
     def _delete_from_schema(self, engine):
         if self.test.BUILD_SCHEMA:
-            super(OpportunisticSqlFixture, self)._delete_from_schema(engine)
+            super()._delete_from_schema(engine)
 
     def _init_resources(self):
         testresources.setUpResources(
@@ -173,17 +172,14 @@ class OpportunisticSqlFixture(lib_fixtures.SqlFixture):
                 ('schema', schema_resource),
                 ('db', database_resource)
             ]
-        else:
-            return [
-                ('db', database_resource)
-            ]
+        return [('db', database_resource)]
 
 
-class BaseSqlTestCase(object):
+class BaseSqlTestCase:
     BUILD_SCHEMA = True
 
     def setUp(self):
-        super(BaseSqlTestCase, self).setUp()
+        super().setUp()
 
         self._setup_database_fixtures()
 
@@ -204,7 +200,7 @@ class SqlTestCase(BaseSqlTestCase, base.BaseTestCase):
     """regular sql test"""
 
 
-class OpportunisticDBTestMixin(object):
+class OpportunisticDBTestMixin:
     """Mixin that converts a BaseSqlTestCase to use the
     OpportunisticSqlFixture.
     """
@@ -223,8 +219,7 @@ class OpportunisticDBTestMixin(object):
             msg = "backend '%s' unavailable" % self.DRIVER
             if self.SKIP_ON_UNAVAILABLE_DB:
                 self.skipTest(msg)
-            else:
-                self.fail(msg)
+            self.fail(msg)
 
     _schema_resources = {}
     _database_resources = {}
@@ -260,15 +255,6 @@ class MySQLTestCaseMixin(OpportunisticDBTestMixin):
     DRIVER = "mysql"
 
 
-class PostgreSQLTestCaseMixin(OpportunisticDBTestMixin):
-    """Mixin that turns any BaseSqlTestCase into a PostgresSQL test suite.
-
-    If the PostgreSQL db is unavailable then this test is skipped, unless
-    OS_FAIL_ON_MISSING_DEPS is enabled.
-    """
-    DRIVER = "postgresql"
-
-
 def module_load_tests(loader, found_tests, pattern):
     """Apply OptimisingTestSuite on a per-module basis.
 
@@ -290,7 +276,7 @@ class WebTestCase(SqlTestCase):
     fmt = 'json'
 
     def setUp(self):
-        super(WebTestCase, self).setUp()
+        super().setUp()
         json_deserializer = wsgi.JSONDeserializer()
         self._deserializers = {
             'application/json': json_deserializer,
@@ -307,7 +293,7 @@ class WebTestCase(SqlTestCase):
         return result
 
 
-class SubDictMatch(object):
+class SubDictMatch:
 
     def __init__(self, sub_dict):
         self.sub_dict = sub_dict
